@@ -1,7 +1,8 @@
+#if (TK_MAJOR_VERSION == 3)
 /*
  * bltBell.c --
  *
- * Copyright 1993-1994 by AT&T Bell Laboratories.
+ * Copyright 1993-1996 by AT&T Bell Laboratories.
  * Permission to use, copy, modify, and distribute this software
  * and its documentation for any purpose and without fee is hereby
  * granted, provided that the above copyright notice appear in all
@@ -20,11 +21,10 @@
  * performance of this software.
  *
  */
-#include "blt.h"
 
-#ifndef BELL_VERSION
-#define BELL_VERSION "1.0"
-#endif
+#include "bltInt.h"
+
+#define BELL_VERSION  "1.0 ($Id$)"
 
 /*
  *----------------------------------------------------------------------
@@ -79,21 +79,9 @@ int
 Blt_BellInit(interp)
     Tcl_Interp *interp;
 {
-    Tk_Window tkwin;
+    static Blt_CmdSpec cmdSpec = { "bell", BELL_VERSION, BellCmd, };
 
-    if (Blt_FindCmd(interp, "blt_bell", (ClientData *)NULL) == TCL_OK) {
-	Tcl_AppendResult(interp, "\"blt_bell\" command already exists",
-	    (char *)NULL);
-	return TCL_ERROR;
-    }
-    tkwin = Tk_MainWindow(interp);
-    if (tkwin == NULL) {
-	Tcl_AppendResult(interp, "\"blt_bell\" requires Tk", (char *)NULL);
-	return TCL_ERROR;
-    }
-    Tcl_SetVar2(interp, "blt_versions", "blt_bell", BELL_VERSION,
-	TCL_GLOBAL_ONLY);
-    Tcl_CreateCommand(interp, "blt_bell", BellCmd, (ClientData)tkwin,
-	(Tcl_CmdDeleteProc *)NULL);
-    return TCL_OK;
+    return (Blt_InitCmd(interp, &cmdSpec));
 }
+
+#endif /* TK_MAJOR_VERSION == 3 */
