@@ -19,7 +19,7 @@
  * without express or implied warranty.
  *
  *
- * Copyright 1993-1994 by AT&T Bell Laboratories.
+ * Copyright 1993-1996 by AT&T Bell Laboratories.
  * Permission to use, copy, modify, and distribute this software
  * and its documentation for any purpose and without fee is hereby
  * granted, provided that the above copyright notice appear in all
@@ -37,37 +37,71 @@
  * tortuous action, arising out of or in connection with the use or
  * performance of this software.
  *
- * "blt_bitmap" command created by George Howlett.
+ * "bitmap" command created by George Howlett.
  */
 
-#include "blt.h"
+#include "bltInt.h"
 #include <ctype.h>
 #include <X11/Xutil.h>
 
-#ifndef BITMAP_VERSION
-#define BITMAP_VERSION "1.1"
-#endif
+#define BITMAP_VERSION "2.0"
+
+extern Tk_CustomOption bltPadOption;
 
 #define MAX_SIZE 255
-enum Formats {
-    UNKNOWN, V10, V11
-};
 
 typedef struct {
-    double theta;		/* Rotation of text string */
+    double rotate;		/* Rotation of text string */
+    double scale;		/* Scaling factor */
     XFontStruct *fontPtr;	/* Font pointer */
-
+    Tk_Justify justify;		/* Justify text */
+    Pad padX, padY;		/* Padding around the text */
 } BitmapInfo;
 
-#define DEF_BITMAP_FONT		"*-Helvetica-Bold-R-Normal-*-14-*"
-#define DEF_BITMAP_ROTATE	"0.0"
+typedef struct {
+    int width, height;		/* Dimension of image */
+    unsigned char *dataArr;	/* Data array for bitmap image */
+    int bytesPerLine;		/* Number of bytes per scan line */
+    int arraySize;		/* Number of bytes in data array */
 
-static Tk_ConfigSpec configSpecs[] =
+} BitmapData;
+
+#define DEF_BITMAP_FONT		STD_FONT
+#define DEF_BITMAP_PAD		"4"
+#define DEF_BITMAP_ROTATE	"0.0"
+#define DEF_BITMAP_SCALE	"1.0"
+#define DEF_BITMAP_JUSTIFY	"center"
+
+static Tk_ConfigSpec composeConfigSpecs[] =
 {
     {TK_CONFIG_FONT, "-font", (char *)NULL, (char *)NULL,
 	DEF_BITMAP_FONT, Tk_Offset(BitmapInfo, fontPtr), 0},
+    {TK_CONFIG_JUSTIFY, "-justify", (char *)NULL, (char *)NULL,
+	DEF_BITMAP_JUSTIFY, Tk_Offset(BitmapInfo, justify),
+	TK_CONFIG_DONT_SET_DEFAULT},
+    {TK_CONFIG_CUSTOM, "-padx", (char *)NULL, (char *)NULL,
+	DEF_BITMAP_PAD, Tk_Offset(BitmapInfo, padX),
+	TK_CONFIG_DONT_SET_DEFAULT, &bltPadOption},
+    {TK_CONFIG_CUSTOM, "-pady", (char *)NULL, (char *)NULL,
+	DEF_BITMAP_PAD, Tk_Offset(BitmapInfo, padY),
+	TK_CONFIG_DONT_SET_DEFAULT, &bltPadOption},
     {TK_CONFIG_DOUBLE, "-rotate", (char *)NULL, (char *)NULL,
-	DEF_BITMAP_ROTATE, Tk_Offset(BitmapInfo, theta),
+	DEF_BITMAP_ROTATE, Tk_Offset(BitmapInfo, rotate),
+	TK_CONFIG_DONT_SET_DEFAULT},
+    {TK_CONFIG_DOUBLE, "-scale", (char *)NULL, (char *)NULL,
+	DEF_BITMAP_SCALE, Tk_Offset(BitmapInfo, scale),
+	TK_CONFIG_DONT_SET_DEFAULT},
+    {TK_CONFIG_END, (char *)NULL, (char *)NULL, (char *)NULL,
+	(char *)NULL, 0, 0}
+};
+
+static Tk_ConfigSpec defineConfigSpecs[] =
+{
+    {TK_CONFIG_DOUBLE, "-rotate", (char *)NULL, (char *)NULL,
+	DEF_BITMAP_ROTATE, Tk_Offset(BitmapInfo, rotate),
+	TK_CONFIG_DONT_SET_DEFAULT},
+    {TK_CONFIG_DOUBLE, "-scale", (char *)NULL, (char *)NULL,
+	DEF_BITMAP_SCALE, Tk_Offset(BitmapInfo, scale),
 	TK_CONFIG_DONT_SET_DEFAULT},
     {TK_CONFIG_END, (char *)NULL, (char *)NULL, (char *)NULL,
 	(char *)NULL, 0, 0}
@@ -149,11 +183,6 @@ static unsigned char bigblt_bits[] =
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
 };
 
-/* Forward declarations */
-extern Pixmap Blt_CreateTextBitmap _ANSI_ARGS_((Display * display,
-	Drawable draw, XFontStruct *fontPtr, char *textStr, double theta,
-	unsigned int *bmWidthPtr, unsigned int *bmHeightPtr));
-
 /*
  *--------------------------------------------------------------
  *
@@ -188,22 +217,16 @@ InitHexTable()
     hexTable['7'] = 7;
     hexTable['8'] = 8;
     hexTable['9'] = 9;
-    hexTable['A'] = 10;
-    hexTable['B'] = 11;
-    hexTable['C'] = 12;
-    hexTable['D'] = 13;
-    hexTable['E'] = 14;
-    hexTable['F'] = 15;
-    hexTable['a'] = 10;
-    hexTable['b'] = 11;
-    hexTable['c'] = 12;
-    hexTable['d'] = 13;
-    hexTable['e'] = 14;
-    hexTable['f'] = 15;
+    hexTable['a'] = hexTable['A'] = 10;
+    hexTable['b'] = hexTable['B'] = 11;
+    hexTable['c'] = hexTable['C'] = 12;
+    hexTable['d'] = hexTable['D'] = 13;
+    hexTable['e'] = hexTable['E'] = 14;
+    hexTable['f'] = hexTable['F'] = 15;
 }
 
 /*
- *--------------------------------------------------------------
+ * -----------------------------------------------------------------------
  *
  * GetHexValue --
  *
@@ -218,7 +241,7 @@ InitHexTable()
  * 	If the conversion fails, interp->result is filled with an
  *	error message.
  *
- *--------------------------------------------------------------
+ * -----------------------------------------------------------------------
  */
 static int
 GetHexValue(interp, string, valuePtr)
@@ -255,29 +278,28 @@ GetHexValue(interp, string, valuePtr)
 }
 
 /*
- *--------------------------------------------------------------
+ * -----------------------------------------------------------------------
  *
- * BitmapToSource --
+ * BitmapToData --
  *
- *	Converts a bitmap into an array of source data.
+ *	Converts a bitmap into an data array.
  *
  * Results:
- *	Returns the number of bytes in an array of source data
- *	representing the bitmap.  Returns -1 if the array could not
- *	be allocated.
+ *	Returns the number of bytes in an data array representing
+ * 	the bitmap.  Returns -1 if the array could not be allocated.
  *
  * Side Effects:
- *	Memory is allocated for source array. Caller must free
+ *	Memory is allocated for the data array. Caller must free
  *	array later.
  *
- *--------------------------------------------------------------
+ * -----------------------------------------------------------------------
  */
 static int
-BitmapToSource(interp, tkwin, bitmap, width, height, dataPtrPtr)
+BitmapToData(interp, tkwin, bitmap, width, height, dataPtrPtr)
     Tcl_Interp *interp;		/* Interpreter to report results to */
     Tk_Window tkwin;		/* Main window of interpreter */
     Pixmap bitmap;		/* Bitmap to be queried */
-    unsigned int width, height;	/* Dimensions of the bitmap */
+    int width, height;		/* Dimensions of the bitmap */
     unsigned char **dataPtrPtr;	/* Pointer to converted array of data */
 {
     int value, bitMask;
@@ -287,7 +309,7 @@ BitmapToSource(interp, tkwin, bitmap, width, height, dataPtrPtr)
     XImage *imagePtr;
     unsigned char *dataPtr;
 
-    /* then convert the bitmap to an image */
+    /* Convert the bitmap to an image */
     imagePtr = XGetImage(Tk_Display(tkwin), bitmap, 0, 0, width, height, 1L,
 	ZPixmap);
 
@@ -298,7 +320,7 @@ BitmapToSource(interp, tkwin, bitmap, width, height, dataPtrPtr)
     arraySize = height * bytes_per_line;
     dataPtr = (unsigned char *)malloc(sizeof(unsigned char) * arraySize);
     if (dataPtr == NULL) {
-	interp->result = "can't allocate array of source data";
+	interp->result = "can't allocate bitmap data array";
 	return -1;
     }
     count = 0;
@@ -326,11 +348,11 @@ BitmapToSource(interp, tkwin, bitmap, width, height, dataPtrPtr)
 }
 
 /*
- *--------------------------------------------------------------
+ * -----------------------------------------------------------------------
  *
- * AsciiToSource --
+ * AsciiToData --
  *
- *	Converts a Tcl list of ASCII values into a source array.
+ *	Converts a Tcl list of ASCII values into a data array.
  *
  * Results:
  *	A standard TCL result.
@@ -339,15 +361,15 @@ BitmapToSource(interp, tkwin, bitmap, width, height, dataPtrPtr)
  * 	If an error occurs while processing the data, interp->result
  *	is filled with a corresponding error message.
  *
- *--------------------------------------------------------------
+ * -----------------------------------------------------------------------
  */
 static int
-AsciiToSource(interp, elemList, width, height, dataPtrPtr)
+AsciiToData(interp, elemList, width, height, dataPtrPtr)
     Tcl_Interp *interp;		/* Interpreter to report results to */
     char *elemList;		/* List of of hex numbers representing
-				 * bitmap source data */
-    unsigned int width, height;	/* Height and width */
-    unsigned char **dataPtrPtr;	/* source data array (output) */
+				 * bitmap data */
+    int width, height;		/* Height and width */
+    unsigned char **dataPtrPtr;	/* data array (output) */
 {
     int arraySize;		/* Number of bytes of data */
     int value;			/* from an input line */
@@ -355,7 +377,9 @@ AsciiToSource(interp, elemList, width, height, dataPtrPtr)
     int bytes_per_line;		/* per scanline of data */
     unsigned char *dataPtr;
     register int count;
-    enum Formats format;
+    enum Formats {
+	V10, V11
+    } format;
     register int i;		/*  */
     char **valueArr;
     int numValues;
@@ -387,8 +411,7 @@ AsciiToSource(interp, elemList, width, height, dataPtrPtr)
 	    arraySize = bytes_per_line * height;
 	}
     }
-    dataPtr = (unsigned char *)
-	calloc(sizeof(unsigned char), (unsigned int)arraySize);
+    dataPtr = (unsigned char *)calloc(sizeof(unsigned char), arraySize);
     if (dataPtr == NULL) {
 	interp->result = "can't allocate memory for bitmap";
 	goto error;
@@ -415,11 +438,203 @@ AsciiToSource(interp, elemList, width, height, dataPtrPtr)
 }
 
 /*
- *--------------------------------------------------------------
+ * -----------------------------------------------------------------------
+ *
+ * RotateData --
+ *
+ *	Creates a new data array of the rotated image.
+ *
+ * Results:
+ *	A standard Tcl result. If the bitmap data is rotated successfully,
+ *	TCL_OK is returned.  But if memory could not be allocated for
+ *	the new data array, TCL_ERROR is returned and an error message
+ *	is left in interp->result.
+ *
+ * Side Effects:
+ *	Memory is allocated for rotated data array. Caller must
+ *	free array later.
+ *
+ * -----------------------------------------------------------------------
+ */
+static int
+RotateData(interp, srcPtr, theta, destPtr)
+    Tcl_Interp *interp;		/* Interpreter to report results to */
+    BitmapData *srcPtr;
+    double theta;		/* Rotate bitmap this many degrees */
+    BitmapData *destPtr;
+{
+    register int dx, dy, sx, sy;
+    double srcX, srcY, destX, destY;	/* Origins of source and destination
+					 * bitmaps */
+    double sinTheta, cosTheta;
+    double transX, transY, rotX, rotY;
+    double radians;
+    unsigned char *dataArr;
+    int arraySize;
+    int pixel, arrIndex;
+
+    srcPtr->bytesPerLine = (srcPtr->width + 7) / 8;
+    Blt_GetBoundingBox(srcPtr->width, srcPtr->height, theta,
+	&(destPtr->width), &(destPtr->height), (XPoint *)NULL);
+
+    destPtr->bytesPerLine = (destPtr->width + 7) / 8;
+    arraySize = destPtr->height * destPtr->bytesPerLine;
+    dataArr = (unsigned char *)calloc(arraySize, sizeof(unsigned char));
+    if (dataArr == NULL) {
+	interp->result = "can't allocate bitmap data array";
+	return TCL_ERROR;
+    }
+    destPtr->dataArr = dataArr;
+    destPtr->arraySize = arraySize;
+
+    radians = (theta / 180.0) * M_PI;
+    sinTheta = sin(radians);
+    cosTheta = cos(radians);
+
+    /*
+     * Coordinates of the centers of the source and destination rectangles
+     */
+    srcX = srcPtr->width * 0.5;
+    srcY = srcPtr->height * 0.5;
+    destX = destPtr->width * 0.5;
+    destY = destPtr->height * 0.5;
+
+    /*
+     * Rotate each pixel of dest image, placing results in source image
+     */
+    for (dx = 0; dx < destPtr->width; dx++) {
+	for (dy = 0; dy < destPtr->height; dy++) {
+	    if (theta == 270.0) {
+		sx = dy, sy = destPtr->width - dx - 1;
+	    } else if (theta == 180.0) {
+		sx = destPtr->width - dx - 1, sy = destPtr->height - dy - 1;
+	    } else if (theta == 90.0) {
+		sx = destPtr->height - dy - 1, sy = dx;
+	    } else if (theta == 0.0) {
+		sx = dx, sy = dy;
+	    } else {
+		/* Translate origin to center of destination image */
+
+		transX = dx - destX;
+		transY = dy - destY;
+
+		/* Rotate the coordinates about the origin */
+
+		rotX = (transX * cosTheta) - (transY * sinTheta);
+		rotY = (transX * sinTheta) + (transY * cosTheta);
+
+		/* Translate back to the center of the source image */
+		rotX += srcX;
+		rotY += srcY;
+
+		sx = BLT_RND(rotX);
+		sy = BLT_RND(rotY);
+
+		/*
+		 * Verify the coordinates, since the destination image can be
+		 * bigger than the source
+		 */
+
+		if ((sx >= srcPtr->width) || (sx < 0) ||
+		    (sy >= srcPtr->height) || (sy < 0)) {
+		    continue;
+		}
+	    }
+	    arrIndex = (srcPtr->bytesPerLine * sy) + (sx / 8);
+	    pixel = srcPtr->dataArr[arrIndex] & (1 << (sx % 8));
+	    if (pixel) {
+		arrIndex = (destPtr->bytesPerLine * dy) + (dx / 8);
+		dataArr[arrIndex] |= (1 << (dx % 8));
+	    }
+	}
+    }
+    return TCL_OK;
+}
+
+/*
+ * -----------------------------------------------------------------------
+ *
+ * ScaleData --
+ *
+ *	Scale the data of the bitmap, creating a new data array of the
+ *	scaled bitmap.
+ *
+ * Results:
+ *	A standard Tcl result. If the bitmap data is scaled successfully,
+ *	TCL_OK is returned.  But if memory could not be allocated for
+ *	the new data array, TCL_ERROR is returned and an error message
+ *	is left in interp->result.
+ *
+ * Side Effects:
+ *	Memory is allocated for scaled data array. Caller must
+ *	dispose of this array later.
+ *
+ * -----------------------------------------------------------------------
+ */
+static int
+ScaleData(interp, srcPtr, scale, destPtr)
+    Tcl_Interp *interp;		/* Interpreter to report results to */
+    BitmapData *srcPtr;
+    double scale;		/* Scale bitmap by this factor */
+    BitmapData *destPtr;
+{
+    register int dx, dy, sx, sy;
+    double scaleX, scaleY;
+    unsigned char *dataArr;
+    int arraySize;
+    int pixel, arrIndex;
+
+    destPtr->width = (int)((srcPtr->width * scale) + 0.5);
+    destPtr->height = (int)((srcPtr->height * scale) + 0.5);
+    srcPtr->bytesPerLine = (srcPtr->width + 7) / 8;
+    destPtr->bytesPerLine = (destPtr->width + 7) / 8;
+
+    arraySize = destPtr->height * destPtr->bytesPerLine;
+    dataArr = (unsigned char *)calloc(arraySize, sizeof(unsigned char));
+    if (dataArr == NULL) {
+	interp->result = "can't allocate bitmap data array";
+	return TCL_ERROR;
+    }
+    destPtr->dataArr = dataArr;
+    destPtr->arraySize = arraySize;
+
+    /*
+     * Scale each pixel of destination image from results of source image
+     */
+    for (dx = 0; dx < destPtr->width; dx++) {
+	for (dy = 0; dy < destPtr->height; dy++) {
+	    scaleX = (dx / scale);
+	    scaleY = (dy / scale);
+
+	    sx = BLT_RND(scaleX);
+	    sy = BLT_RND(scaleY);
+
+	    /*
+	     * Verify the coordinates, since the destination image can be
+	     * bigger than the source
+	     */
+
+	    if ((sx >= srcPtr->width) || (sx < 0) ||
+		(sy >= srcPtr->height) || (sy < 0)) {
+		continue;
+	    }
+	    arrIndex = (srcPtr->bytesPerLine * sy) + (sx / 8);
+	    pixel = srcPtr->dataArr[arrIndex] & (1 << (sx % 8));
+	    if (pixel) {
+		arrIndex = (destPtr->bytesPerLine * dy) + (dx / 8);
+		dataArr[arrIndex] |= (1 << (dx % 8));
+	    }
+	}
+    }
+    return TCL_OK;
+}
+
+/*
+ * -----------------------------------------------------------------------
  *
  * ShowBitmap --
  *
- *	Returns a list of hex values corresponding to the source
+ *	Returns a list of hex values corresponding to the data
  *	bits of the bitmap given.
  *
  *	Converts the unsigned character value into a two character
@@ -428,10 +643,10 @@ AsciiToSource(interp, elemList, width, height, dataPtrPtr)
  *	already output.
  *
  * Results:
- *	Returns TCL_ERROR is a source data array can't be generated
+ *	Returns TCL_ERROR if a data array can't be generated
  *	from the bitmap (memory allocation failure), otherwise TCL_OK.
  *
- *--------------------------------------------------------------
+ * -----------------------------------------------------------------------
  */
 static int
 ShowBitmap(interp, tkwin, bitmap)
@@ -444,11 +659,11 @@ ShowBitmap(interp, tkwin, bitmap)
     int arraySize;
     register int i;
     char string[200];
-    unsigned int width, height;
+    int width, height;
 
     /* Get the dimensions of the bitmap */
     Tk_SizeOfBitmap(Tk_Display(tkwin), bitmap, &width, &height);
-    arraySize = BitmapToSource(interp, tkwin, bitmap, width, height, &dataPtr);
+    arraySize = BitmapToData(interp, tkwin, bitmap, width, height, &dataPtr);
     if (arraySize < 0) {
 	return TCL_ERROR;
     }
@@ -465,14 +680,14 @@ ShowBitmap(interp, tkwin, bitmap)
 /*
  *--------------------------------------------------------------
  *
- * ComposeBitmap --
+ * ComposeOper --
  *
  *	Converts the text string into an internal bitmap.
  *
  *	There's a lot of extra (read unnecessary) work going on
  *	here, but I don't (right now) think that it matters much.
  *	The rotated bitmap (formerly an image) is converted back
- *	to an image just so we can convert it to source data for
+ *	to an image just so we can convert it to a data array for
  *	Tk_DefineBitmap.
  *
  * Results:
@@ -485,55 +700,83 @@ ShowBitmap(interp, tkwin, bitmap)
  *--------------------------------------------------------------
  */
 static int
-ComposeBitmap(tkwin, interp, argc, argv)
+ComposeOper(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
     int argc;			/* Number of arguments */
     char **argv;		/* Argument list */
 {
-    unsigned int width, height;	/* dimensions of bitmap */
+    int width, height;		/* dimensions of bitmap */
     Pixmap bitmap;		/* Text bitmap */
-    unsigned char *dataPtr;	/* Copy of source array from bitmap */
+    unsigned char *dataPtr;	/* Data array derived from text bitmap */
     int arraySize;
     BitmapInfo info;		/* Text rotation and font information */
+    int result;
+    double theta;
+    TextAttributes textAttr;
 
-    if (argc < 4) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-	    " compose name text ?flags?\"", (char *)NULL);
-	return TCL_ERROR;
-    }
     /* Initialize info and process flags */
     info.fontPtr = NULL;	/* Will be initialized by Tk_ConfigureWidget */
-    info.theta = 0.0;		/* No rotation by default */
-    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc - 4, argv + 4,
-	    (char *)&info, 0) != TCL_OK) {
+    info.rotate = 0.0;		/* No rotation by default */
+    info.scale = 1.0;
+    info.justify = TK_JUSTIFY_CENTER;
+    info.padLeft = info.padRight = 0;
+    info.padTop = info.padBottom = 0;
+    if (Tk_ConfigureWidget(interp, tkwin, composeConfigSpecs,
+	    argc - 4, argv + 4, (char *)&info, 0) != TCL_OK) {
 	return TCL_ERROR;
     }
-    /* Need a window Id to create the bitmap.  Could use the root window. */
-    Tk_MakeWindowExist(tkwin);
-    bitmap = Blt_CreateTextBitmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
-	info.fontPtr, argv[3], info.theta, &width, &height);
-    Tk_FreeOptions(configSpecs, (char *)&info, Tk_Display(tkwin), 0);
+    theta = BLT_FMOD(info.rotate, (double)360.0);
+    if (theta < 0.0) {
+	theta += 360.0;
+    }
+    textAttr.fontPtr = info.fontPtr;
+    textAttr.theta = theta;
+    textAttr.justify = info.justify;
+    textAttr.padX = info.padX;
+    textAttr.padY = info.padY;
 
-    /* Convert bitmap back to source data */
-    arraySize = BitmapToSource(interp, tkwin, bitmap, width, height, &dataPtr);
-    XFreePixmap(Tk_Display(tkwin), bitmap);
+    bitmap = Blt_CreateTextBitmap(tkwin, argv[3], &textAttr, &width, &height);
+
+    /* Free the font structure, since we don't need it anymore */
+    Tk_FreeOptions(composeConfigSpecs, (char *)&info, Tk_Display(tkwin), 0);
+
+    /* Convert bitmap back to a data array */
+    arraySize = BitmapToData(interp, tkwin, bitmap, width, height, &dataPtr);
+    Tk_FreePixmap(Tk_Display(tkwin), bitmap);
     if (arraySize < 0) {
 	return TCL_ERROR;
     }
-    /* Create the bitmap again, this time using Tk's bitmap facilities */
-    if (Tk_DefineBitmap(interp, Tk_GetUid(argv[2]), (char *)dataPtr,
-	    width, height) != TCL_OK) {
-	free((char *)dataPtr);
-	return TCL_ERROR;
+    if (info.scale != 1.0) {
+	BitmapData srcData, destData;
+
+	srcData.dataArr = dataPtr;
+	srcData.width = width;
+	srcData.height = height;
+	srcData.arraySize = arraySize;
+
+	result = ScaleData(interp, &srcData, info.scale, &destData);
+	free((char *)dataPtr);	/* Free the un-scaled data array */
+	if (result != TCL_OK) {
+	    return TCL_ERROR;
+	}
+	dataPtr = destData.dataArr;
+	width = destData.width;
+	height = destData.height;
     }
-    return TCL_OK;
+    /* Create the bitmap again, this time using Tk's bitmap facilities */
+    result = Tk_DefineBitmap(interp, Tk_GetUid(argv[2]), (char *)dataPtr,
+	width, height);
+    if (result != TCL_OK) {
+	free((char *)dataPtr);
+    }
+    return result;
 }
 
 /*
  *--------------------------------------------------------------
  *
- * DefineBitmap --
+ * DefineOper --
  *
  *	Converts the dataList into an internal bitmap.
  *
@@ -548,34 +791,38 @@ ComposeBitmap(tkwin, interp, argc, argv)
  */
 /* ARGSUSED */
 static int
-DefineBitmap(tkwin, interp, argc, argv)
+DefineOper(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
     int argc;			/* Number of arguments */
     char **argv;		/* Argument list */
 {
-    unsigned int width, height;	/* dimensions of bitmap */
+    int width, height;		/* dimensions of bitmap */
     int w, h;			/* Dimensions from input */
     char **dimArr, **elemArr;
     int numDim, numElem;
     unsigned char *dataPtr;	/* working variable */
     register char *p;
-    BitmapInfo info;		/* TBA: Text rotation and font information */
+    BitmapInfo info;		/* not used */
+    int arraySize;
+    int result;
+    double theta;
 
-    if (argc != 4) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-	    " define name data\"", (char *)NULL);
+    /* Initialize info and then process flags */
+    info.rotate = 0.0;		/* No rotation by default */
+    info.scale = 1.0;		/* No scaling by default */
+    if (Tk_ConfigureWidget(interp, tkwin, defineConfigSpecs,
+	    argc - 4, argv + 4, (char *)&info, 0) != TCL_OK) {
 	return TCL_ERROR;
     }
-    /* Initialize info and process flags */
-
+    dataPtr = NULL;
     dimArr = elemArr = NULL;
     if (Tcl_SplitList(interp, argv[3], &numElem, &elemArr) != TCL_OK) {
 	return TCL_ERROR;
     }
     if (numElem != 2) {
 	Tcl_AppendResult(interp, "wrong # of bitmap data components: ",
-	    "should be \"dimList bitsList\"", (char *)NULL);
+	    "should be \"dimensions sourceData\"", (char *)NULL);
 	goto error;
     }
     if (Tcl_SplitList(interp, elemArr[0], &numDim, &dimArr) != TCL_OK) {
@@ -595,8 +842,8 @@ DefineBitmap(tkwin, interp, argc, argv)
 	    "\"", (char *)NULL);
 	goto error;
     }
-    width = (unsigned int)w;
-    height = (unsigned int)h;
+    width = w;
+    height = h;
     free((char *)dimArr);
     dimArr = NULL;
 
@@ -607,19 +854,63 @@ DefineBitmap(tkwin, interp, argc, argv)
 	    *p = ' ';
 	}
     }
-    if (AsciiToSource(interp, elemArr[1], width, height, &dataPtr) < 0) {
+    arraySize = AsciiToData(interp, elemArr[1], width, height, &dataPtr);
+    if (arraySize < 0) {
 	goto error;
     }
     free((char *)elemArr);
+    elemArr = NULL;
 
-    /* If bitmap is to be rotated or scale, do it here */
-
-    if (Tk_DefineBitmap(interp, Tk_GetUid(argv[2]), (char *)dataPtr,
-	    width, height) == TCL_OK) {
-	return TCL_OK;
+    theta = BLT_FMOD(info.rotate, 360.0);
+    if (theta < 0.0) {
+	theta += 360.0;
     }
-    free((char *)dataPtr);
+    /* If bitmap is to be rotated or scale, do it here */
+    if (theta != 0.0) {
+	BitmapData srcData, destData;
+
+	srcData.dataArr = dataPtr;
+	srcData.width = width;
+	srcData.height = height;
+	srcData.arraySize = arraySize;
+
+	result = RotateData(interp, &srcData, theta, &destData);
+	free((char *)dataPtr);	/* Free the un-rotated data array */
+	if (result != TCL_OK) {
+	    return TCL_ERROR;
+	}
+	dataPtr = destData.dataArr;
+	width = destData.width;
+	height = destData.height;
+    }
+    if (info.scale != 1.0) {
+	BitmapData srcData, destData;
+
+	srcData.dataArr = dataPtr;
+	srcData.width = width;
+	srcData.height = height;
+	srcData.arraySize = arraySize;
+
+	result = ScaleData(interp, &srcData, info.scale, &destData);
+	free((char *)dataPtr);	/* Free the un-scaled data array */
+	if (result != TCL_OK) {
+	    return TCL_ERROR;
+	}
+	dataPtr = destData.dataArr;
+	width = destData.width;
+	height = destData.height;
+    }
+    result = Tk_DefineBitmap(interp, Tk_GetUid(argv[2]), (char *)dataPtr,
+	width, height);
+    if (result != TCL_OK) {
+	free((char *)dataPtr);
+    }
+    return result;
+
   error:
+    if (dataPtr != NULL) {
+	free((char *)dataPtr);
+    }
     if (dimArr != NULL) {
 	free((char *)dimArr);
     }
@@ -632,26 +923,22 @@ DefineBitmap(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * BitmapExists --
+ * ExistOper --
  *
  *	Indicates if the named bitmap exists.
  *
  *--------------------------------------------------------------
  */
+/*ARGSUSED*/
 static int
-BitmapExists(tkwin, interp, argc, argv)
+ExistsOper(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
-    int argc;			/* Number of arguments */
+    int argc;			/* not used */
     char **argv;		/* Argument list */
 {
     Pixmap bitmap;
 
-    if (argc != 3) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-	    " exists name\"", (char *)NULL);
-	return TCL_ERROR;
-    }
     bitmap = Tk_GetBitmap(interp, tkwin, Tk_GetUid(argv[2]));
     Tcl_ResetResult(interp);
     if (bitmap == None) {
@@ -666,27 +953,23 @@ BitmapExists(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * BitmapHeight --
+ * HeightOper --
  *
  *	Returns the height of the named bitmap.
  *
  *--------------------------------------------------------------
  */
+/*ARGSUSED*/
 static int
-BitmapHeight(tkwin, interp, argc, argv)
+HeightOper(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
-    int argc;			/* Number of arguments */
+    int argc;			/* not used */
     char **argv;		/* Argument list */
 {
-    unsigned int width, height;
+    int width, height;
     Pixmap bitmap;
 
-    if (argc != 3) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-	    " height name\"", (char *)NULL);
-	return TCL_ERROR;
-    }
     bitmap = Tk_GetBitmap(interp, tkwin, Tk_GetUid(argv[2]));
     if (bitmap == None) {
 	return TCL_ERROR;
@@ -700,27 +983,23 @@ BitmapHeight(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * BitmapWidth --
+ * WidthOper --
  *
  *	Returns the width of the named bitmap.
  *
  *--------------------------------------------------------------
  */
+/*ARGSUSED*/
 static int
-BitmapWidth(tkwin, interp, argc, argv)
+WidthOper(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
-    int argc;			/* Number of arguments */
+    int argc;			/* not used */
     char **argv;		/* Argument list */
 {
-    unsigned int width, height;
+    int width, height;
     Pixmap bitmap;
 
-    if (argc != 3) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-	    " width name\"", (char *)NULL);
-	return TCL_ERROR;
-    }
     bitmap = Tk_GetBitmap(interp, tkwin, Tk_GetUid(argv[2]));
     if (bitmap == None) {
 	return TCL_ERROR;
@@ -734,32 +1013,30 @@ BitmapWidth(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * BitmapBits --
+ * SourceOper --
  *
- *	Returns the source data (excluding width and height)
+ *	Returns the data array (excluding width and height)
  *	of the named bitmap.
  *
  *--------------------------------------------------------------
  */
+/*ARGSUSED*/
 static int
-BitmapBits(tkwin, interp, argc, argv)
+SourceOper(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
-    int argc;			/* Number of arguments */
+    int argc;			/* not used */
     char **argv;		/* Argument list */
 {
     Pixmap bitmap;
 
-    if (argc != 3) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-	    " bits name\"", (char *)NULL);
-	return TCL_ERROR;
-    }
     bitmap = Tk_GetBitmap(interp, tkwin, Tk_GetUid(argv[2]));
     if (bitmap == None) {
 	return TCL_ERROR;
     }
-    ShowBitmap(interp, tkwin, bitmap);
+    if (ShowBitmap(interp, tkwin, bitmap) != TCL_OK) {
+	return TCL_ERROR;
+    }
     Tk_FreeBitmap(Tk_Display(tkwin), bitmap);
     return TCL_OK;
 }
@@ -767,29 +1044,25 @@ BitmapBits(tkwin, interp, argc, argv)
 /*
  *--------------------------------------------------------------
  *
- * BitmapData --
+ * DataOper --
  *
- *	Returns the source data (including width and height)
+ *	Returns the data array, including width and height,
  *	of the named bitmap.
  *
  *--------------------------------------------------------------
  */
+/*ARGSUSED*/
 static int
-BitmapData(tkwin, interp, argc, argv)
+DataOper(tkwin, interp, argc, argv)
     Tk_Window tkwin;		/* Main window of interpreter */
     Tcl_Interp *interp;		/* Interpreter to report results to */
-    int argc;			/* Number of arguments */
+    int argc;			/* not used */
     char **argv;		/* Argument list */
 {
     Pixmap bitmap;
     char string[200];
-    unsigned int width, height;
+    int width, height;
 
-    if (argc != 3) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-	    " data name\"", (char *)NULL);
-	return TCL_ERROR;
-    }
     bitmap = Tk_GetBitmap(interp, tkwin, Tk_GetUid(argv[2]));
     if (bitmap == None) {
 	return TCL_ERROR;
@@ -797,12 +1070,43 @@ BitmapData(tkwin, interp, argc, argv)
     Tk_SizeOfBitmap(Tk_Display(tkwin), bitmap, &width, &height);
     sprintf(string, "%d %d", width, height);
     Tcl_AppendResult(interp, "  {", string, "} {", (char *)NULL);
-    ShowBitmap(interp, tkwin, bitmap);
+    if (ShowBitmap(interp, tkwin, bitmap) != TCL_OK) {
+	return TCL_ERROR;
+    }
     Tcl_AppendResult(interp, "\n  }", (char *)NULL);
     Tk_FreeBitmap(Tk_Display(tkwin), bitmap);
     return TCL_OK;
 }
 
+/*
+ *--------------------------------------------------------------
+ *
+ * BLT Sub-command specification:
+ *
+ *	- Name of the sub-command.
+ *	- Minimum number of characters needed to unambiguously
+ *        recognize the sub-command.
+ *	- Pointer to the function to be called for the sub-command.
+ *	- Minimum number of arguments accepted.
+ *	- Maximum number of arguments accepted.
+ *	- String to be displayed for usage.
+ *
+ *--------------------------------------------------------------
+ */
+static Blt_OperSpec operSpecs[] =
+{
+    {"compose", 1, (Blt_OperProc) ComposeOper, 4, 0,
+	"bitmapName text ?flags?",},
+    {"data", 2, (Blt_OperProc) DataOper, 3, 3, "bitmapName",},
+    {"define", 2, (Blt_OperProc) DefineOper, 4, 0,
+	"bitmapName data ?flags?",},
+    {"exists", 1, (Blt_OperProc) ExistsOper, 3, 3, "bitmapName",},
+    {"height", 1, (Blt_OperProc) HeightOper, 3, 3, "bitmapName",},
+    {"source", 1, (Blt_OperProc) SourceOper, 3, 3, "bitmapName",},
+    {"width", 1, (Blt_OperProc) WidthOper, 3, 3, "bitmapName",},
+};
+static int numSpecs = sizeof(operSpecs) / sizeof(Blt_OperSpec);
+
 /*
  *--------------------------------------------------------------
  *
@@ -820,6 +1124,7 @@ BitmapData(tkwin, interp, argc, argv)
  *
  *--------------------------------------------------------------
  */
+
 static int
 BitmapCmd(clientData, interp, argc, argv)
     ClientData clientData;	/* Main window of interpreter */
@@ -828,38 +1133,16 @@ BitmapCmd(clientData, interp, argc, argv)
     char **argv;
 {
     Tk_Window tkwin = (Tk_Window)clientData;
-    int length;
-    char c;
+    Blt_OperProc proc;
+    int result;
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-	    " option name ?args?\"", (char *)NULL);
-	return TCL_ERROR;
-    }
-    c = argv[1][0];
-    length = strlen(argv[1]);
-    if ((c == 'd') && (length > 1) &&
-	(strncmp(argv[1], "define", length) == 0)) {
-	return (DefineBitmap(tkwin, interp, argc, argv));
-    } else if ((c == 'e') && (strncmp(argv[1], "exists", length) == 0)) {
-	return (BitmapExists(tkwin, interp, argc, argv));
-    } else if ((c == 'w') && (strncmp(argv[1], "width", length) == 0)) {
-	return (BitmapWidth(tkwin, interp, argc, argv));
-    } else if ((c == 'h') && (strncmp(argv[1], "height", length) == 0)) {
-	return (BitmapHeight(tkwin, interp, argc, argv));
-    } else if ((c == 'b') && (strncmp(argv[1], "bits", length) == 0)) {
-	return (BitmapBits(tkwin, interp, argc, argv));
-    } else if ((c == 'd') && (length > 1) &&
-	(strncmp(argv[1], "data", length) == 0)) {
-	return (BitmapData(tkwin, interp, argc, argv));
-    } else if ((c == 'c') && (strncmp(argv[1], "compose", length) == 0)) {
-	return (ComposeBitmap(tkwin, interp, argc, argv));
-    } else {
-	Tcl_AppendResult(interp, "bad option \"", argv[1], "\": should be ",
-	    " define, exists, width, height, data, bits, or compose",
-	    (char *)NULL);
+    proc = Blt_LookupOperation(interp, numSpecs, operSpecs, BLT_OPER_ARG1,
+	argc, argv);
+    if (proc == NULL) {
 	return TCL_ERROR;
     }
+    result = (*proc) (tkwin, interp, argc, argv);
+    return (result);
 }
 
 /*
@@ -882,25 +1165,13 @@ int
 Blt_BitmapInit(interp)
     Tcl_Interp *interp;
 {
-    Tk_Window tkwin;
-
-    if (Blt_FindCmd(interp, "blt_bitmap", (ClientData *)NULL) == TCL_OK) {
-	Tcl_AppendResult(interp, "\"blt_bitmap\" command already exists",
-	    (char *)NULL);
-	return TCL_ERROR;
-    }
-    tkwin = Tk_MainWindow(interp);
-    if (tkwin == NULL) {
-	Tcl_AppendResult(interp, "\"blt_bitmap\" requires Tk", (char *)NULL);
-	return TCL_ERROR;
-    }
-    Tcl_SetVar2(interp, "blt_versions", "blt_bitmap", BITMAP_VERSION,
-	TCL_GLOBAL_ONLY);
-    Tcl_CreateCommand(interp, "blt_bitmap", BitmapCmd, (ClientData)tkwin,
-	(Tcl_CmdDeleteProc *)NULL);
+    static Blt_CmdSpec cmdSpec = { "bitmap", BITMAP_VERSION, BitmapCmd, };
 
     /* Define the BLT logo bitmaps */
 
+    if (Blt_InitCmd(interp, &cmdSpec) != TCL_OK) {
+	return TCL_ERROR;
+    }
     Tk_DefineBitmap(interp, Tk_GetUid("BLT"), (char *)blt_bits,
 	blt_width, blt_height);
     Tk_DefineBitmap(interp, Tk_GetUid("bigBLT"), (char *)bigblt_bits,
