@@ -27,9 +27,9 @@
  *  AUTHOR:  Michael J. McLennan       Phone: (215)770-2842
  *           AT&T Bell Laboratories   E-mail: aluxpo!mmc@att.com
  *
- *     RCS:  bltDragDrop.c,v 1.8 1994/01/18 21:29:51 gah Exp
+ *     RCS:  $Id: bltDragDrop.c,v 1.10 1994/04/14 21:11:03 gah Exp $
  * ========================================================================
- *                 Copyright (c) 1993-1994  AT&T Bell Laboratories
+ *                 Copyright (c) 1993-1996  AT&T Bell Laboratories
  * ========================================================================
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -49,42 +49,28 @@
  * performance of this software.
  * ========================================================================
  */
-#include "blt.h"
+#include "bltInt.h"
 #include <X11/Xatom.h>
 
-#ifndef DRAGDROP_VERSION
 #define DRAGDROP_VERSION "2.1"
-#endif
+static char dragDropCmd[] = "drag&drop";
 
-#define DRAGDROP_COMMAND	"blt_drag&drop"   /* Command name */
-#define DRAGDROP_CLASS		"DragDrop"  	  /* CLASS NAME for token window */
-#define DRAGDROP_PROPINFO	"BltDragDropInfo" /* Property name */
-#define DRAGDROP_ERRORPROC	"tkerror"		  /* Error Proc used to report 
-											   * drag&drop background errors */
 
-#ifndef _BLT_H
-/*
- *  -------------  The following color definitions are taken directly
- *  >> WARNING <<  from "default.h" in the Tk distribution, and should
- *  -------------  be kept up to date with those conventions.
- */
-#define BLACK       "Black"
-#define WHITE       "White"
-#define GRAY        "#b0b0b0"
-
-#define BISQUE1     "#ffe4c4"
-#define BISQUE2     "#eed5b7"
-#define BISQUE3     "#cdb79e"
-#endif							/* _BLT_H */
-#define DEF_BUTTON_ACTIVE_BG_COLOR  BISQUE2
-#define DEF_BUTTON_ACTIVE_BG_MONO   BLACK
-#define DEF_BUTTON_ACTIVE_FG_COLOR  BLACK
-#define DEF_BUTTON_ACTIVE_FG_MONO   WHITE
-#define DEF_BUTTON_BG_COLOR     BISQUE1
-#define DEF_BUTTON_BG_MONO      WHITE
+#define DEF_BUTTON_ACTIVE_BG_COLOR  STD_ACTIVE_BG_COLOR
+#define DEF_BUTTON_ACTIVE_BG_MONO   STD_ACTIVE_BG_MONO
+#define DEF_BUTTON_ACTIVE_FG_COLOR  STD_ACTIVE_FG_COLOR
+#define DEF_BUTTON_ACTIVE_FG_MONO   STD_ACTIVE_FG_MONO
+#define DEF_BUTTON_BG_COLOR         STD_NORMAL_BG_COLOR
+#define DEF_BUTTON_BG_MONO          STD_NORMAL_BG_MONO
 #define DEF_TOKEN_OUTLINE_COLOR     BLACK
 #define DEF_TOKEN_OUTLINE_MONO      BLACK
 
+/* Error Proc used to report drag&drop background errors */
+#define DEF_ERROR_PROC 		    "tkerror"  
+
+static char className[] = "DragDrop"; /* CLASS NAME for token window */
+static char propName[] = "DragDropInfo"; /* Property name */
+
 /*
  *  DRAG&DROP ROOT WINDOW HIERARCHY (cached during "drag" operations)
  */
@@ -140,7 +126,7 @@ typedef struct {
 
 	Tk_Window tokenwin;           /* window representing drag item */
 	Tk_Anchor tokenAnchor;        /* position of token win relative to mouse */
-	Cursor tokenCursor;           /* cursor used when dragging token */
+	Tk_Cursor tokenCursor;	      /* cursor used when dragging token */
 	Tk_3DBorder tokenOutline;     /* outline around token window */
 	Tk_3DBorder tokenBorder;      /* border/background for token window */
 	int tokenBorderWidth;         /* border width in pixels */
@@ -160,7 +146,7 @@ typedef struct {
 	int overTargetWin;            /* non-zero => over target window */
 	int tokenx, tokeny;           /* last position of token window */
 	Tk_TimerToken hidetoken;      /* token for routine to hide tokenwin */
-	Cursor normalCursor;          /* cursor restored after dragging */
+	Tk_Cursor normalCursor;       /* cursor restored after dragging */
 
 	char *send;                   /* list of data handler names or "all" */
 	DD_SourceHndl *handlers;      /* list of data handlers */
@@ -376,11 +362,6 @@ static void StackDelete _ANSI_ARGS_((DD_Stack *stack));
 static void StackPush _ANSI_ARGS_((ClientData cdata, DD_Stack *stack));
 static ClientData StackPop _ANSI_ARGS_((DD_Stack *stack));
 
-#ifndef sun
-extern char *strdup _ANSI_ARGS_((CONST char *s));
-#endif
-
-
 
 /*
  * ------------------------------------------------------------------------
@@ -393,56 +374,32 @@ extern char *strdup _ANSI_ARGS_((CONST char *s));
  */
 int
 Blt_DragDropInit(interp)
-	Tcl_Interp *interp;  /* interpreter to be updated */
+    Tcl_Interp *interp;  /* interpreter to be updated */
 {
-	DD_RegList *ddlist;
-	Tk_Window tkwin;
-	Tcl_CmdInfo cmdInfo;
-
-	/*
-	 *  See if drag&drop is already installed.
-	 */
-	if (Tcl_GetCommandInfo(interp, DRAGDROP_COMMAND, &cmdInfo))
-	{
-		Tcl_ResetResult(interp);
-		Tcl_AppendResult(interp, "already installed: ", DRAGDROP_COMMAND,
-						 (char*)NULL);
-		return TCL_ERROR;
-	}
-
-	/*
-	 *  Make sure that this is a Tk application.
-	 */
-	tkwin = Tk_MainWindow(interp);
-	if (tkwin == None)
-	{
-		Tcl_ResetResult(interp);
-		Tcl_AppendResult(interp, "requires Tk facilities: ", DRAGDROP_COMMAND,
-						 (char*)NULL);
-		return TCL_ERROR;
-	}
-
-	/*
-	 *  Install drag&drop facilities.
-	 */
+    DD_RegList *ddlist;
+    static Blt_CmdSpec cmdSpec = { 
+	"drag&drop", DRAGDROP_VERSION, DragDrop_Cmd, DragDrop_Delete, 
+    };
+    /*
+     *  Install drag&drop facilities.
+     */
     ddlist = (DD_RegList*)malloc(sizeof(DD_RegList));
     ddlist->interp = interp;
-    ddlist->root = tkwin;
-    Tcl_InitHashTable(&ddlist->srcList,TCL_STRING_KEYS);
-    Tcl_InitHashTable(&ddlist->trgList,TCL_STRING_KEYS);
-    ddlist->errorProc = strdup(DRAGDROP_ERRORPROC);
+    ddlist->root = Tk_MainWindow(interp);
+    Tcl_InitHashTable(&ddlist->srcList,	TCL_STRING_KEYS);
+    Tcl_InitHashTable(&ddlist->trgList,	TCL_STRING_KEYS);
+    ddlist->errorProc = strdup(DEF_ERROR_PROC);
     ddlist->numactive = 0;
     ddlist->locx = ddlist->locy = 0;
     ddlist->pool = NULL;
 
-	Tcl_CreateCommand(interp, DRAGDROP_COMMAND, DragDrop_Cmd,
-		(ClientData)ddlist, DragDrop_Delete);
-
-	Tcl_SetVar2(interp, "blt_versions", DRAGDROP_COMMAND, DRAGDROP_VERSION,
-		TCL_GLOBAL_ONLY);
-
-	return TCL_OK;
+    cmdSpec.clientData = (ClientData)ddlist;
+    if (Blt_InitCmd(interp, &cmdSpec) != TCL_OK) {
+	return TCL_ERROR;
+    }
+    return TCL_OK;
 }
+
 
 /*
  * ------------------------------------------------------------------------
@@ -454,22 +411,22 @@ Blt_DragDropInit(interp)
  */
 static void
 DragDrop_Delete(cdata)
-	ClientData cdata;    /* client data for drag&drop command */
+    ClientData cdata;    /* client data for drag&drop command */
 {
-	DD_RegList *ddlist = (DD_RegList*)cdata;
-	DD_WinRep *wrpool, *wrnext;
-
+    DD_RegList *ddlist = (DD_RegList*)cdata;
+    DD_WinRep *wrpool, *wrnext;
+    
     Tcl_DeleteHashTable(&ddlist->srcList);
     Tcl_DeleteHashTable(&ddlist->trgList);
-	if (ddlist->errorProc != NULL) {
-		free((char*)ddlist->errorProc);
-	}
-	for (wrpool=ddlist->pool; wrpool; wrpool=wrnext)
-	{
-		wrnext = wrpool->next;
-		free((char*)wrpool);
-	}
-	free((char*)ddlist);
+    
+    if (ddlist->errorProc != NULL) {
+	free((char*)ddlist->errorProc);
+    }
+    for (wrpool=ddlist->pool; wrpool; wrpool=wrnext) {
+	wrnext = wrpool->next;
+	free((char*)wrpool);
+    }
+    free((char*)ddlist);
 }
 
 /*
@@ -520,7 +477,7 @@ DragDrop_Cmd(clientData, interp, argc, argv)
 	if (argc < 2)
 	{
 		Tcl_AppendResult(interp, "wrong # args: should be \"",
-			argv[0], " option ?args?\"",
+			argv[0], " oper ?args?\"",
 			(char*)NULL);
 		return TCL_ERROR;
 	}
@@ -566,7 +523,7 @@ DragDrop_Cmd(clientData, interp, argc, argv)
 			return TCL_ERROR;
 		}
 		dsPtr->display = Tk_Display(dsPtr->tkwin);
-		dsPtr->ddAtom  = XInternAtom(dsPtr->display, DRAGDROP_PROPINFO, False);
+		dsPtr->ddAtom  = XInternAtom(dsPtr->display, propName, False);
 
 		if (newEntry)
 			ConfigSource(interp, dsPtr, 0, (char**)NULL, 0);
@@ -644,7 +601,7 @@ DragDrop_Cmd(clientData, interp, argc, argv)
 			}
 			else
 			{
-				Tcl_AppendResult(interp, "bad option \"", argv[3],
+				Tcl_AppendResult(interp, "bad operation \"", argv[3],
 					"\": must be config or handler",
 					(char*)NULL);
 				return TCL_ERROR;
@@ -667,7 +624,7 @@ DragDrop_Cmd(clientData, interp, argc, argv)
 				DestroySourceInfo(ddlist,argv[2]);
 				return TCL_ERROR;
 			}
-			Tk_SetClass(tokenwin, DRAGDROP_CLASS);
+			Tk_SetClass(tokenwin, className);
 			Tk_CreateEventHandler(tokenwin, ExposureMask|StructureNotifyMask,
 				DDTokenEventProc, (ClientData)dsPtr);
 
@@ -685,9 +642,9 @@ DragDrop_Cmd(clientData, interp, argc, argv)
 					"bind %.100s <ButtonPress-%d> {%s drag %.100s %%X %%Y}; \
 					bind %.100s <B%d-Motion> {%s drag %.100s %%X %%Y}; \
 					bind %.100s <ButtonRelease-%d> {%s drop %.100s %%X %%Y}",
-					argv[2], dsPtr->button, DRAGDROP_COMMAND, argv[2],
-					argv[2], dsPtr->button, DRAGDROP_COMMAND, argv[2],
-					argv[2], dsPtr->button, DRAGDROP_COMMAND, argv[2]);
+					argv[2], dsPtr->button, dragDropCmd, argv[2],
+					argv[2], dsPtr->button, dragDropCmd, argv[2],
+					argv[2], dsPtr->button, dragDropCmd, argv[2]);
 
 				if (Tcl_Eval(interp, buffer) != TCL_OK)
 				{
@@ -1117,7 +1074,7 @@ DragDrop_Cmd(clientData, interp, argc, argv)
 	 */
 	else
 	{
-		Tcl_AppendResult(interp, "bad option \"", argv[1], "\": must be source, target, drag, drop, errors, active or location",
+		Tcl_AppendResult(interp, "bad operation \"", argv[1], "\": must be source, target, drag, drop, errors, active or location",
 			(char*)NULL);
 		return TCL_ERROR;
 	}
@@ -1164,8 +1121,8 @@ GetSourceInfo(ddlist,pathname,newEntry)
 		dsPtr->rejectFg = NULL;
 		dsPtr->rejectBg = NULL;
 		dsPtr->rejectSt = None;
-		dsPtr->rejectFgGC = None;
-		dsPtr->rejectBgGC = None;
+		dsPtr->rejectFgGC = NULL;
+		dsPtr->rejectBgGC = NULL;
 
 		dsPtr->pkgcmdInProg = 0;
 		dsPtr->pkgcmd = NULL;
@@ -1217,9 +1174,9 @@ DestroySourceInfo(ddlist,pathname)
 
 		Tk_FreeOptions(SourceConfigSpecs, (char *)dsPtr, dsPtr->display, 0);
 
-		if (dsPtr->rejectFgGC != None) 
+		if (dsPtr->rejectFgGC != NULL) 
 			Tk_FreeGC(dsPtr->display, dsPtr->rejectFgGC);
-		if (dsPtr->rejectBgGC != None) 
+		if (dsPtr->rejectBgGC != NULL) 
 			Tk_FreeGC(dsPtr->display, dsPtr->rejectBgGC);
 		if (dsPtr->pkgcmdResult)	   
 			free(dsPtr->pkgcmdResult);
@@ -1295,7 +1252,7 @@ ConfigSource(interp, dsPtr, argc, argv, flags)
 	}
 	newGC = Tk_GetGC(dsPtr->tkwin, gcMask, &gcValues);
 
-	if (dsPtr->rejectFgGC != None)
+	if (dsPtr->rejectFgGC != NULL)
 		Tk_FreeGC(dsPtr->display, dsPtr->rejectFgGC);
 	dsPtr->rejectFgGC = newGC;
 
@@ -1309,7 +1266,7 @@ ConfigSource(interp, dsPtr, argc, argv, flags)
 
 	newGC = Tk_GetGC(dsPtr->tkwin, gcMask, &gcValues);
 
-	if (dsPtr->rejectBgGC != None)
+	if (dsPtr->rejectBgGC != NULL)
 		Tk_FreeGC(dsPtr->display, dsPtr->rejectBgGC);
 	dsPtr->rejectBgGC = newGC;
 
@@ -1695,7 +1652,7 @@ DragDropSend(dsPtr)
 
 		sprintf(buffer, "%d %d", dsPtr->tokenx, dsPtr->tokeny);
 		status = Tcl_VarEval(ddlist->interp,
-			"send {",target->ddinterp,"} ", DRAGDROP_COMMAND," location ",buffer,
+			"send {",target->ddinterp,"} ", dragDropCmd," location ",buffer,
 			(char*)NULL);
 
 		if (status == TCL_OK)
@@ -1767,7 +1724,7 @@ DragDropSendHndlr(dsPtr,interpName,ddName)
 	 */
 	Tcl_ResetResult(interp); /* for Tcl_AppendResult() below */
 	if (Tcl_VarEval(interp,
-		"send {",interpName,"} ", DRAGDROP_COMMAND," target {",ddName,"} handler",
+		"send {",interpName,"} ", dragDropCmd," target {",ddName,"} handler",
 		(char*)NULL) != TCL_OK)
 		return NULL;
 
@@ -2204,10 +2161,12 @@ AddDDProp(dtPtr)
 	char buffer[MAX_PROP_SIZE], *path, *info;
 	DD_TargetHndl *thandl;
 
-	if (dtPtr->tkwin != None)
+	if (dtPtr->tkwin != NULL)
 	{
+		static char command[] = { "winfo name ." };
+
 		path = Tk_PathName(dtPtr->tkwin);
-		if (Tcl_Eval(interp,"winfo name .")==TCL_OK)
+		if (Tcl_Eval(interp, command)==TCL_OK)
 			sprintf(buffer, "%s]%s]", interp->result, path);
 		else
 			sprintf(buffer, "]%s]", path);
@@ -2216,7 +2175,7 @@ AddDDProp(dtPtr)
 		for (thandl=dtPtr->handlers; thandl; thandl=thandl->next)
 			Tcl_AppendResult(interp, thandl->dataType, " ", (char*)NULL);
 
-		ddProperty = XInternAtom(dtPtr->display, DRAGDROP_PROPINFO, False);
+		ddProperty = XInternAtom(dtPtr->display, propName, False);
 		info = interp->result;
 
 		XChangeProperty(dtPtr->display, Tk_WindowId(dtPtr->tkwin),
@@ -2248,6 +2207,7 @@ DDTokenEventProc(clientData, eventPtr)
 			int bd;
 
 			bd = dsPtr->tokenBorderWidth;
+#if (TK_MAJOR_VERSION == 3)
 			Tk_Fill3DRectangle(dsPtr->display, Tk_WindowId(tkwin),
 				dsPtr->tokenOutline,
 				0, 0, Tk_Width(tkwin), Tk_Height(tkwin),
@@ -2256,6 +2216,16 @@ DDTokenEventProc(clientData, eventPtr)
 				dsPtr->tokenBorder,
 				bd, bd, Tk_Width(tkwin)-2*bd, Tk_Height(tkwin)-2*bd,
 				bd, (dsPtr->overTargetWin) ? TK_RELIEF_RAISED : TK_RELIEF_FLAT);
+#else
+			Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
+				dsPtr->tokenOutline,
+				0, 0, Tk_Width(tkwin), Tk_Height(tkwin),
+				0, TK_RELIEF_FLAT);
+			Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
+				dsPtr->tokenBorder,
+				bd, bd, Tk_Width(tkwin)-2*bd, Tk_Height(tkwin)-2*bd,
+				bd, (dsPtr->overTargetWin) ? TK_RELIEF_RAISED : TK_RELIEF_FLAT);
+#endif /* TK_MAJOR_VERSION == 3 */
 		}
 	}
 	else if (eventPtr->type == DestroyNotify)
@@ -2366,6 +2336,7 @@ UpdateDDToken(clientData)
 	bd = dsPtr->tokenBorderWidth;
 	if (dsPtr->overTargetWin ^ status)
 	{
+#if (TK_MAJOR_VERSION == 3)
 		Tk_Fill3DRectangle(dsPtr->display, Tk_WindowId(tkwin),
 			dsPtr->tokenOutline,
 			0, 0, Tk_Width(tkwin), Tk_Height(tkwin),
@@ -2374,7 +2345,16 @@ UpdateDDToken(clientData)
 			dsPtr->tokenBorder,
 			bd, bd, Tk_Width(tkwin)-2*bd, Tk_Height(tkwin)-2*bd,
 			bd, (status) ? TK_RELIEF_RAISED : TK_RELIEF_FLAT);
-
+#else
+		Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
+			dsPtr->tokenOutline,
+			0, 0, Tk_Width(tkwin), Tk_Height(tkwin),
+			0, TK_RELIEF_FLAT);
+		Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
+			dsPtr->tokenBorder,
+			bd, bd, Tk_Width(tkwin)-2*bd, Tk_Height(tkwin)-2*bd,
+			bd, (status) ? TK_RELIEF_RAISED : TK_RELIEF_FLAT);
+#endif /* TK_MAJOR_VERSION == 3 */
 		/*
 		 *  If the source has a site command, then invoke it to
 		 *  modify the appearance of the token window.  Pass any
@@ -2432,7 +2412,7 @@ static void
 RejectDDToken(dsPtr)
 	DD_Source* dsPtr;  /* widget data */
 {
-	int div = 6;      /* controls size of rejection symbol */
+	int divisor = 6;      /* controls size of rejection symbol */
 	int w,h,lwid,x,y,margin;
 
 	Tk_Window tkwin = dsPtr->tokenwin;
@@ -2441,10 +2421,10 @@ RejectDDToken(dsPtr)
 	w = Tk_Width(tkwin) - 2*margin;
 	h = Tk_Height(tkwin) - 2*margin;
 
-	lwid = (w < h) ? w/div : h/div;
+	lwid = (w < h) ? w/divisor : h/divisor;
 	lwid = (lwid < 1) ? 1 : lwid;
 
-	w = h = lwid*(div-1);
+	w = h = lwid*(divisor-1);
 	x = (Tk_Width(tkwin) - w)/2;
 	y = (Tk_Height(tkwin) - h)/2;
 
