@@ -2,7 +2,7 @@
 /*
  * bltGrElem.h --
  *
- * Copyright 1991-1994 by AT&T Bell Laboratories.
+ * Copyright 1991-1996 by AT&T Bell Laboratories.
  * Permission to use, copy, modify, and distribute this software
  * and its documentation for any purpose and without fee is hereby
  * granted, provided that the above copyright notice appear in all
@@ -25,28 +25,12 @@
 #ifndef _ELEMENT_H
 #define _ELEMENT_H
 
-#define DEF_ACTIVE_SIZE 8	/* Default size of the static active
-				 * array.  Contains indices of the
-				 * active data points within an element */
-
-
-typedef enum {
-    ELEM_NORMAL, ELEM_ACTIVE
-} ElementActiveStates;
-
-/*
- * An element can be a line or a bar.
- */
-typedef enum {
-    LINE_ELEM_TYPE, BAR_ELEM_TYPE
-} ElementClassType;
-
 /*
  * It can have one of the following types of symbol.
  */
 typedef enum {
-    LINE_SYMBOL, SQUARE_SYMBOL, CIRCLE_SYMBOL, DIAMOND_SYMBOL,
-    PLUS_SYMBOL, CROSS_SYMBOL, SPLUS_SYMBOL, SCROSS_SYMBOL
+    SYMBOL_NONE, SYMBOL_SQUARE, SYMBOL_CIRCLE, SYMBOL_DIAMOND,
+    SYMBOL_PLUS, SYMBOL_CROSS, SYMBOL_SPLUS, SYMBOL_SCROSS, SYMBOL_TRIANGLE
 } SymbolType;
 
 /*
@@ -56,33 +40,48 @@ typedef enum {
  */
 typedef struct Vector {
     double *data;		/* Array of values */
-    unsigned int length;	/* Number of entries in the array */
-    unsigned int size;		/* Size of data array allocated */
+    int length;			/* Number of entries in the array */
     double min, max;		/* Smallest/largest values in the array */
-    double logMin;		/* Smallest positive value */
+    Blt_VectorId clientId;	/* If non-NULL, a client token identifying
+				 * the external vector. */
+    Element *elemPtr;		/* Element vector is associated with */
 } Vector;
 
+#define SetVisible(status) ((status) |= DONT_CHECK_VISIBLE)
+#define SetClipped(status) ((status) &= ~DONT_CHECK_VISIBLE)
+#define SetActive(status)  ((status) |= DONT_CHECK_ACTIVE)
+#define SetNormal(status)  ((status) &= ~DONT_CHECK_ACTIVE)
+
+#define IsVisible(status) ((status) & DONT_CHECK_VISIBLE)
+#define IsClipped(status) (((status) & DONT_CHECK_VISIBLE) == 0)
+#define IsActive(status)  ((status) & DONT_CHECK_ACTIVE)
+#define IsNormal(status)  (((status) & DONT_CHECK_ACTIVE) == 0)
+
+typedef unsigned char DataStatus;
+
+#define DONT_CHECK_VISIBLE	(1<<0)
+#define DONT_CHECK_ACTIVE	(1<<1)
+#define DATA_OK			(DONT_CHECK_VISIBLE|DONT_CHECK_ACTIVE)
+
+#define	DrawVisible(status)	(((status) & DONT_CHECK_VISIBLE) == 0)
+#define	DrawActive(status)	(((status) & DONT_CHECK_ACTIVE) == 0)
 /*
  * An element has one or more vectors plus several attributes,
  * such as line style, thickness, color, and symbol type.  It
  * has an identifier which distinguishes it among the list of all
  * elements.
  */
-typedef struct Element Element;
-typedef struct ClosestPoint ClosestPoint;
+typedef struct ClosestInfo ClosestInfo;
 
-typedef void (ElemDisplayProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr,
-	int active));
-typedef void (ElemPrintProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr,
-	int active));
-typedef void (ElemDestroyProc) _ANSI_ARGS_((Graph *graphPtr,
-	Element *elemPtr));
+typedef void (ElemDrawProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr));
+typedef void (ElemPrintProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr));
+typedef void (ElemDestroyProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr));
 typedef int (ElemConfigProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr));
-typedef void (ElemLayoutProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr));
+typedef void (ElemCoordsProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr));
 typedef int (ElemLimitsProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr,
 	GraphAxis *axisPtr, double *minPtr, double *maxPtr));
-typedef int (ElemDistanceProc) _ANSI_ARGS_((Graph *graphPtr,
-	Element *elemPtr, int x, int y, ClosestPoint * closePtr));
+typedef void (ElemClosestProc) _ANSI_ARGS_((Graph *graphPtr, Element *elemPtr,
+	ClosestInfo *infoPtr));
 typedef void (ElemDrawSymbolsProc) _ANSI_ARGS_((Graph *graphPtr,
 	Element *elemPtr, int symbolSize, XPoint *pointArr, int numPoints,
 	int active));
@@ -91,53 +90,54 @@ typedef void (ElemPrintSymbolsProc) _ANSI_ARGS_((Graph *graphPtr,
 	int active));
 
 struct Element {
-    Tcl_Interp *interp;		/* Interpreter of graph widget */
-    ElementClassType type;	/* Type of element; either BAR_ELEMENT or
+    Graph *graphPtr;		/* Graph widget of element*/
+    ElementType type;		/* Type of element; either BAR_ELEMENT or
 				 * LINE_ELEMENT */
-    unsigned int flags;
-    Tk_Uid id;			/* Identifier to refer the element. Used in
+    unsigned int flags;		/* Indicates if the entire element is active,
+				 * or if coordinates need to be calculated */
+    Tk_Uid nameId;		/* Identifier to refer the element. Used in
 				 * the "insert", "delete", or "show",
 				 * commands. */
     int mapped;			/* If true, element is currently visible. */
     Tk_ConfigSpec *configSpecs;	/* Configuration specifications */
     char *label;		/* Label displayed in legend */
-    SymbolType symbol;		/* Element symbol type */
-    double symbolScale;		/* Size of symbol as a percentage of the
-				 * drawing area. */
-    unsigned int symbolSize;	/* Computed size of symbol in pixels. */
-
+    unsigned int axisMask;	/* Indicates which axes to map element's
+				 * coordinates onto */
     Vector x, y;		/* Contains array of floating point graph
 				 * coordinate values. Also holds min/max and
 				 * the number of coordinates */
-    unsigned int axisFlags;	/* Indicates which axes to map element's
-				 * coordinates onto */
-    int *activeArr;		/* Array of indices of active data
-				 * points (malloc-ed). Initially points
-				 * to static storage "staticArr". */
-    int staticArr[DEF_ACTIVE_SIZE];
-    int numActivePoints;	/* Number of active data points. If
-				 * zero and active bit is set in
-				 * "flags", then all data points are
-				 * active. */
+    DataStatus *statusArr;	/* Array of flags (malloc-ed) which
+				 * indicate which data points are active
+				 * (drawn with "active" colors) and visible. */
+    int arraySize;		/* Number of active data points. Special
+				 * case: if numActivePoints==0 and the active
+				 * bit is set in "flags", then all data
+				 * points are drawn active. */
 
     ElemConfigProc *configProc;
     ElemDestroyProc *destroyProc;
-    ElemDisplayProc *displayProc;
+    ElemDrawProc *drawNormalProc;
+    ElemDrawProc *drawActiveProc;
     ElemLimitsProc *limitsProc;
-    ElemDistanceProc *closestProc;
-    ElemLayoutProc *layoutProc;
-    ElemPrintProc *printProc;
+    ElemClosestProc *closestProc;
+    ElemCoordsProc *coordsProc;
+    ElemPrintProc *printNormalProc;
+    ElemPrintProc *printActiveProc;
     ElemDrawSymbolsProc *drawSymbolsProc;
     ElemPrintSymbolsProc *printSymbolsProc;
 };
 
-#define	LAYOUT_NEEDED 	(1<<0)	/* Indicates that the element's
+#define	COORDS_NEEDED 	(1<<0)	/* Indicates that the element's
 				 * configuration has changed such that
 				 * its layout of the element (i.e. its
 				 * position in the graph window) needs
 				 * to be recalculated. */
 
-#define	ACTIVE 		(1<<8)	/* Non-zero indicates that the element
+#define	ELEM_LINE_ACTIVE (1<<7)	/* Non-zero indicates that the element
+				 * should be drawn in its active
+				 * foreground and background
+				 * colors. */
+#define	ELEM_ACTIVE	(1<<8)	/* Non-zero indicates that the element
 				 * should be drawn in its active
 				 * foreground and background
 				 * colors. */
@@ -147,11 +147,23 @@ struct Element {
 				 * should be drawn in its active
 				 * foreground and background
 				 * colors. */
-struct ClosestPoint {
-    Element *elemPtr;		/* Closest element */
-    int index;			/* Index of closest data point */
-    double x, y;		/* Graph coordinates of closest point */
+struct ClosestInfo {
+    int halo;			/* Maximal distance a candidate point can
+				 * be from the sample window coordinate */
+    int interpolate;		/* If non-zero, find the closest point on the
+				 * entire line segment, instead of the closest
+				 * end point. */
+    int x, y;			/* Screen coordinates of sample point */
+
     double dist;		/* Distance from the screen coordinates */
+    Tk_Uid elemId;		/* Id of closest element */
+    GraphPoint closest;		/* Graph coordinates of closest point */
+    int index;			/* Index of closest data point */
 };
 
+extern double Blt_FindVectorMinimum _ANSI_ARGS_((Vector *vecPtr,
+	double minLimit));
+extern void Blt_ResizeStatusArray _ANSI_ARGS_((Element *elemPtr,
+	int numPoints));
+
 #endif /* _ELEMENT_H */
