@@ -4,7 +4,7 @@
  *
  *	This module implements a crosshairs for the graph widget.
  *
- * Copyright 1991-1994 by AT&T Bell Laboratories.
+ * Copyright 1991-1996 by AT&T Bell Laboratories.
  * Permission to use, copy, modify, and distribute this software
  * and its documentation for any purpose and without fee is hereby
  * granted, provided that the above copyright notice appear in all
@@ -25,11 +25,11 @@
  * Graph widget created by Sani Nassif and George Howlett.
  */
 
-#include "blt.h"
 #include "bltGraph.h"
 
-
 extern Tk_CustomOption bltPositionOption;
+extern Tk_CustomOption bltLengthOption;
+extern Tk_CustomOption bltDashesOption;
 
 /*
  * -------------------------------------------------------------------
@@ -54,9 +54,10 @@ typedef struct {
 				 * is not necessarily consistent with the
 				 * internal state variable.  This is true when
 				 * the hot spot is off the graph.  */
-    int dashes;			/* Dashstyle of the crosshairs. This represents
-				 * the number of alternatingly drawn pixels. If
-				 * zero, the hairs are drawn as a solid line */
+    Dashes dashes;		/* Dashstyle of the crosshairs. This represents
+				 * an array of alternatingly drawn pixel
+				 * values. If NULL, the hairs are drawn as a
+				 * solid line */
     int lineWidth;		/* Width of the simulated crosshair lines */
     XSegment segArr[2];		/* Positions of line segments representing the
 				 * simulated crosshairs. */
@@ -65,7 +66,7 @@ typedef struct {
 				 * GXxor to not require redraws of graph */
 } Crosshairs;
 
-#define DEF_HAIRS_DASHES	"0"
+#define DEF_HAIRS_DASHES	(char *)NULL
 #define DEF_HAIRS_FG_COLOR	BLACK
 #define DEF_HAIRS_FG_MONO	BLACK
 #define DEF_HAIRS_LINE_WIDTH	"0"
@@ -77,14 +78,15 @@ static Tk_ConfigSpec configSpecs[] =
     {TK_CONFIG_COLOR, "-color", "color", "Color",
 	DEF_HAIRS_FG_COLOR, Tk_Offset(Crosshairs, colorPtr),
 	TK_CONFIG_COLOR_ONLY},
-    {TK_CONFIG_INT, "-dashes", "xhairsDashes", "Dashes",
-	DEF_HAIRS_DASHES, Tk_Offset(Crosshairs, dashes), 0},
-    {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
+    {TK_CONFIG_COLOR, "-color", "color", "Color",
 	DEF_HAIRS_FG_MONO, Tk_Offset(Crosshairs, colorPtr),
 	TK_CONFIG_MONO_ONLY},
-    {TK_CONFIG_INT, "-linewidth", "xhairsLineWidth", "XhairsLinewidth",
+    {TK_CONFIG_CUSTOM, "-dashes", "xhairsDashes", "Dashes",
+	DEF_HAIRS_DASHES, Tk_Offset(Crosshairs, dashes),
+	TK_CONFIG_NULL_OK, &bltDashesOption},
+    {TK_CONFIG_CUSTOM, "-linewidth", "xhairsLineWidth", "XhairsLinewidth",
 	DEF_HAIRS_LINE_WIDTH, Tk_Offset(Crosshairs, lineWidth),
-	TK_CONFIG_DONT_SET_DEFAULT},
+	TK_CONFIG_DONT_SET_DEFAULT, &bltLengthOption},
     {TK_CONFIG_BOOLEAN, "-mapped", "xhairsMapped", "XhairsMapped",
 	DEF_HAIRS_MAPPED, Tk_Offset(Crosshairs, mapped),
 	TK_CONFIG_DONT_SET_DEFAULT},
@@ -94,7 +96,7 @@ static Tk_ConfigSpec configSpecs[] =
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
 };
 
-extern int Blt_PointOnGraph _ANSI_ARGS_((Graph *, XPoint *));
+extern int Blt_PointOnGraph _ANSI_ARGS_((Graph *, int, int));
 
 /*
  *----------------------------------------------------------------------
@@ -197,35 +199,35 @@ ConfigureCrosshairs(graphPtr)
     gcValues.background = colorValue;
     gcValues.foreground = (colorValue ^ hairsPtr->colorPtr->pixel);
 
-    if (hairsPtr->lineWidth < 0) {
-	hairsPtr->lineWidth = 0;
-    }
-    gcValues.line_width = hairsPtr->lineWidth;
+    gcValues.line_width = LINEWIDTH(hairsPtr->lineWidth);
     gcMask = (GCForeground | GCBackground | GCFunction | GCLineWidth);
-    if (hairsPtr->dashes > 0) {
-	gcValues.dashes = hairsPtr->dashes;
-	gcValues.dash_offset = 0;
+    if (hairsPtr->dashes.numValues > 0) {
 	gcValues.line_style = LineOnOffDash;
-	gcMask |= (GCDashOffset | GCDashList | GCLineStyle);
+	gcMask |= GCLineStyle;
+    }
+    newGC = Blt_GetUnsharedGC(graphPtr->tkwin, gcMask, &gcValues);
+    if (hairsPtr->dashes.numValues > 0) {
+	XSetDashes(graphPtr->display, newGC, 0, hairsPtr->dashes.valueList,
+	    hairsPtr->dashes.numValues);
     }
-    newGC = Tk_GetGC(graphPtr->tkwin, gcMask, &gcValues);
     if (hairsPtr->gc != NULL) {
-	Tk_FreeGC(graphPtr->display, hairsPtr->gc);
+	XFreeGC(graphPtr->display, hairsPtr->gc);
     }
     hairsPtr->gc = newGC;
 
     /*
      * Are the new coordinates on the graph?
      */
-    if (!Blt_PointOnGraph(graphPtr, &(hairsPtr->anchorPos))) {
+    if (!Blt_PointOnGraph(graphPtr, hairsPtr->anchorPos.x,
+	    hairsPtr->anchorPos.y)) {
 	return;			/* Coordinates are off the graph */
     }
     hairsPtr->segArr[0].x2 = hairsPtr->segArr[0].x1 = hairsPtr->anchorPos.x;
-    hairsPtr->segArr[0].y1 = graphPtr->origin.y;
-    hairsPtr->segArr[0].y2 = graphPtr->extreme.y;
+    hairsPtr->segArr[0].y1 = graphPtr->lly;
+    hairsPtr->segArr[0].y2 = graphPtr->ury;
     hairsPtr->segArr[1].y2 = hairsPtr->segArr[1].y1 = hairsPtr->anchorPos.y;
-    hairsPtr->segArr[1].x1 = graphPtr->origin.x;
-    hairsPtr->segArr[1].x2 = graphPtr->extreme.x;
+    hairsPtr->segArr[1].x1 = graphPtr->llx;
+    hairsPtr->segArr[1].x2 = graphPtr->urx;
 
     if (hairsPtr->mapped) {
 	TurnOnHairs(graphPtr->tkwin, hairsPtr);
@@ -280,10 +282,10 @@ UpdateCrosshairs(graphPtr)
 {
     Crosshairs *hairsPtr = (Crosshairs *)graphPtr->crosshairs;
 
-    hairsPtr->segArr[0].y1 = graphPtr->origin.y;
-    hairsPtr->segArr[0].y2 = graphPtr->extreme.y;
-    hairsPtr->segArr[1].x1 = graphPtr->origin.x;
-    hairsPtr->segArr[1].x2 = graphPtr->extreme.x;
+    hairsPtr->segArr[0].y1 = graphPtr->lly;
+    hairsPtr->segArr[0].y2 = graphPtr->ury;
+    hairsPtr->segArr[1].x1 = graphPtr->llx;
+    hairsPtr->segArr[1].x2 = graphPtr->urx;
 }
 
 /*
@@ -307,7 +309,7 @@ DestroyCrosshairs(graphPtr)
 
     Tk_FreeOptions(configSpecs, (char *)hairsPtr, graphPtr->display, 0);
     if (hairsPtr->gc != NULL) {
-	Tk_FreeGC(graphPtr->display, hairsPtr->gc);
+	XFreeGC(graphPtr->display, hairsPtr->gc);
     }
     free((char *)hairsPtr);
 }
@@ -336,11 +338,9 @@ Blt_CreateCrosshairs(graphPtr)
 
     hairsPtr = (Crosshairs *)calloc(1, sizeof(Crosshairs));
     if (hairsPtr == NULL) {
-	graphPtr->interp->result = "can't allocate crosshairs structure";
-	return TCL_ERROR;
+	panic("can't allocate crosshairs structure");
     }
     hairsPtr->mapped = hairsPtr->state = 0;
-    hairsPtr->anchorPos.x = hairsPtr->anchorPos.y = DEF_POSITION;
     hairsPtr->destroyProc = DestroyCrosshairs;
     hairsPtr->toggleProc = ToggleCrosshairs;
     hairsPtr->updateProc = UpdateCrosshairs;
@@ -356,7 +356,182 @@ Blt_CreateCrosshairs(graphPtr)
 /*
  *----------------------------------------------------------------------
  *
- * Blt_CrosshairsCmd --
+ * CgetOper --
+ *
+ *	Queries configuration attributes of the crosshairs such as
+ *	line width, dashes, and position.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ *----------------------------------------------------------------------
+ */
+static int
+CgetOper(graphPtr, interp, argc, argv)
+    Graph *graphPtr;
+    Tcl_Interp *interp;
+    int argc;			/* unused */
+    char **argv;
+{
+    Crosshairs *hairsPtr = (Crosshairs *)graphPtr->crosshairs;
+
+    return (Tk_ConfigureValue(interp, graphPtr->tkwin, configSpecs,
+	    (char *)hairsPtr, argv[3], 0));
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * ConfigureOper --
+ *
+ *	Queries or resets configuration attributes of the crosshairs
+ * 	such as line width, dashes, and position.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side Effects:
+ *	Crosshairs are reset.
+ *
+ *----------------------------------------------------------------------
+ */
+static int
+ConfigureOper(graphPtr, interp, argc, argv)
+    Graph *graphPtr;
+    Tcl_Interp *interp;
+    int argc;
+    char **argv;
+{
+    Crosshairs *hairsPtr = (Crosshairs *)graphPtr->crosshairs;
+
+    if (argc == 3) {
+	return (Tk_ConfigureInfo(interp, graphPtr->tkwin, configSpecs,
+		(char *)hairsPtr, (char *)NULL, 0));
+    } else if (argc == 4) {
+	return (Tk_ConfigureInfo(interp, graphPtr->tkwin, configSpecs,
+		(char *)hairsPtr, argv[3], 0));
+    }
+    if (Tk_ConfigureWidget(interp, graphPtr->tkwin, configSpecs, argc - 3,
+	    argv + 3, (char *)hairsPtr, TK_CONFIG_ARGV_ONLY) != TCL_OK) {
+	return TCL_ERROR;
+    }
+    ConfigureCrosshairs(graphPtr);
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * MapOper --
+ *
+ *	Maps the crosshairs.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side Effects:
+ *	Crosshairs are reset if necessary.
+ *
+ *----------------------------------------------------------------------
+ */
+/*ARGSUSED*/
+static int
+MapOper(graphPtr, interp, argc, argv)
+    Graph *graphPtr;
+    Tcl_Interp *interp;
+    int argc;
+    char **argv;
+{
+    Crosshairs *hairsPtr = (Crosshairs *)graphPtr->crosshairs;
+
+    if (!hairsPtr->mapped) {
+	hairsPtr->mapped = TRUE;
+	TurnOnHairs(graphPtr->tkwin, hairsPtr);
+    }
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * UnmapOper --
+ *
+ *	Unmaps the crosshairs.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side Effects:
+ *	Crosshairs are reset if necessary.
+ *
+ *----------------------------------------------------------------------
+ */
+/*ARGSUSED*/
+static int
+UnmapOper(graphPtr, interp, argc, argv)
+    Graph *graphPtr;
+    Tcl_Interp *interp;
+    int argc;
+    char **argv;
+{
+    Crosshairs *hairsPtr = (Crosshairs *)graphPtr->crosshairs;
+
+    if (hairsPtr->mapped) {
+	hairsPtr->mapped = FALSE;
+	TurnOffHairs(graphPtr->tkwin, hairsPtr);
+    }
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * ToggleOper --
+ *
+ *	Toggles the state of the crosshairs.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side Effects:
+ *	Crosshairs are reset.
+ *
+ *----------------------------------------------------------------------
+ */
+/*ARGSUSED*/
+static int
+ToggleOper(graphPtr, interp, argc, argv)
+    Graph *graphPtr;
+    Tcl_Interp *interp;
+    int argc;
+    char **argv;
+{
+    Crosshairs *hairsPtr = (Crosshairs *)graphPtr->crosshairs;
+
+    hairsPtr->mapped = (!hairsPtr->mapped);
+    if (hairsPtr->mapped) {
+	TurnOnHairs(graphPtr->tkwin, hairsPtr);
+    } else {
+	TurnOffHairs(graphPtr->tkwin, hairsPtr);
+    }
+    return TCL_OK;
+}
+
+
+static Blt_OperSpec operSpecs[] =
+{
+    {"cget", 2, (Blt_OperProc)CgetOper, 4, 4, "option",},
+    {"configure", 2, (Blt_OperProc)ConfigureOper, 3, 0, "?options...?",},
+    {"off", 2, (Blt_OperProc)UnmapOper, 3, 3, "",},
+    {"on", 2, (Blt_OperProc)MapOper, 3, 3, "",},
+    {"toggle", 1, (Blt_OperProc)ToggleOper, 3, 3, "",},
+};
+static int numSpecs = sizeof(operSpecs) / sizeof(Blt_OperSpec);
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_CrosshairsOper --
  *
  *	User routine to configure crosshair simulation.  Crosshairs
  *	are simulated by drawing line segments parallel to both axes
@@ -373,76 +548,19 @@ Blt_CreateCrosshairs(graphPtr)
  *
  *----------------------------------------------------------------------
  */
-/* ARGSUSED */
 int
-Blt_CrosshairsCmd(graphPtr, argc, argv)
+Blt_CrosshairsOper(graphPtr, interp, argc, argv)
     Graph *graphPtr;
+    Tcl_Interp *interp;
     int argc;
     char **argv;
 {
-    char c;
-    int length;
-    int boolean;
-    Tcl_Interp *interp = graphPtr->interp;
-    Crosshairs *hairsPtr = (Crosshairs *)graphPtr->crosshairs;
+    Blt_OperProc proc;
 
-    if (argc < 3) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-	    " crosshairs option ?args?\"", NULL);
+    proc = Blt_LookupOperation(interp, numSpecs, operSpecs, BLT_OPER_ARG2,
+	argc, argv);
+    if (proc == NULL) {
 	return TCL_ERROR;
     }
-    c = argv[2][0];
-    length = strlen(argv[2]);
-
-    if ((c == 'c') && (strncmp(argv[2], "configure", length) == 0)) {
-	if (argc < 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" crosshairs configure \"", NULL);
-	    return TCL_ERROR;
-	}
-	if (argc == 3) {
-	    return (Tk_ConfigureInfo(interp, graphPtr->tkwin, configSpecs,
-		    (char *)hairsPtr, (char *)NULL, 0));
-	} else if (argc == 4) {
-	    return (Tk_ConfigureInfo(interp, graphPtr->tkwin, configSpecs,
-		    (char *)hairsPtr, argv[3], 0));
-	}
-	if (Tk_ConfigureWidget(interp, graphPtr->tkwin, configSpecs, argc - 3,
-		argv + 3, (char *)hairsPtr, TK_CONFIG_ARGV_ONLY) != TCL_OK) {
-	    return TCL_ERROR;
-	}
-	ConfigureCrosshairs(graphPtr);
-	return TCL_OK;
-    } else if ((c == 't') && (strncmp(argv[2], "toggle", length) == 0)) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" crosshairs toggle\"", NULL);
-	    return TCL_ERROR;
-	}
-	boolean = (!hairsPtr->mapped);
-    } else if ((c == 's') && (strncmp(argv[2], "set", length) == 0)) {
-	if (argc != 4) {
-	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" crosshairs set boolean\"", NULL);
-	    return TCL_ERROR;
-	}
-	if (Tcl_GetBoolean(graphPtr->interp, argv[3], &boolean) != TCL_OK) {
-	    return TCL_ERROR;
-	}
-    } else {
-	Tcl_AppendResult(interp, "bad crosshairs option \"", argv[2],
-	    "\": should be configure, set, or toggle", (char *)NULL);
-	return TCL_ERROR;
-    }
-    if (hairsPtr->mapped == boolean) {
-	return TCL_OK;
-    }
-    hairsPtr->mapped = boolean;
-
-    if (hairsPtr->mapped) {
-	TurnOnHairs(graphPtr->tkwin, hairsPtr);
-    } else {
-	TurnOffHairs(graphPtr->tkwin, hairsPtr);
-    }
-    return TCL_OK;
+    return ((*proc) (graphPtr, interp, argc, argv));
 }
