@@ -3,7 +3,7 @@
  *
  *	Generic linked list management routines.
  *
- * Copyright 1991-1994 by AT&T Bell Laboratories.
+ * Copyright 1991-1996 by AT&T Bell Laboratories.
  * Permission to use, copy, modify, and distribute this software
  * and its documentation for any purpose and without fee is hereby
  * granted, provided that the above copyright notice appear in all
@@ -23,12 +23,12 @@
  *
  */
 
-#include "blt.h"
+#include "bltInt.h"
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_InitLinkedList --
+ * Blt_InitList --
  *
  *	Initialized a linked list.
  *
@@ -38,20 +38,20 @@
  *----------------------------------------------------------------------
  */
 void
-Blt_InitLinkedList(listPtr, type)
-    Blt_LinkedList *listPtr;
+Blt_InitList(listPtr, type)
+    Blt_List *listPtr;
     int type;
 {
 
     listPtr->numEntries = 0;
-    listPtr->headPtr = listPtr->tailPtr = (Blt_ListEntry *)NULL;
+    listPtr->headPtr = listPtr->tailPtr = (Blt_ListItem *)NULL;
     listPtr->type = type;
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_CreateLinkedList --
+ * Blt_NewList --
  *
  *	Creates a new linked list structure and initializes its pointers
  *
@@ -60,15 +60,15 @@ Blt_InitLinkedList(listPtr, type)
  *
  *----------------------------------------------------------------------
  */
-Blt_LinkedList *
-Blt_CreateLinkedList(type)
+Blt_List *
+Blt_NewList(type)
     int type;
 {
-    Blt_LinkedList *listPtr;
+    Blt_List *listPtr;
 
-    listPtr = (Blt_LinkedList *)malloc(sizeof(Blt_LinkedList));
+    listPtr = (Blt_List *)malloc(sizeof(Blt_List));
     if (listPtr != NULL) {
-	Blt_InitLinkedList(listPtr, type);
+	Blt_InitList(listPtr, type);
     }
     return (listPtr);
 }
@@ -76,7 +76,7 @@ Blt_CreateLinkedList(type)
 /*
  *----------------------------------------------------------------------
  *
- * Blt_CreateListEntry --
+ * Blt_NewItem --
  *
  *	Creates a list entry holder.  This routine does not insert
  *	the entry into the list, nor does it no attempt to maintain
@@ -92,25 +92,27 @@ Blt_CreateLinkedList(type)
  *
  *----------------------------------------------------------------------
  */
-Blt_ListEntry *
-Blt_CreateListEntry(key)
+Blt_ListItem *
+Blt_NewItem(key)
     char *key;			/* Unique key to reference object */
 {
-    register Blt_ListEntry *entryPtr;
+    register Blt_ListItem *itemPtr;
 
-    entryPtr = (Blt_ListEntry *)malloc(sizeof(Blt_ListEntry));
-    if (entryPtr != (Blt_ListEntry *)NULL) {
-	entryPtr->keyPtr = key;
-	entryPtr->clientData = (ClientData)NULL;
-	entryPtr->nextPtr = entryPtr->prevPtr = (Blt_ListEntry *)NULL;
+    itemPtr = (Blt_ListItem *)malloc(sizeof(Blt_ListItem));
+    if (itemPtr == (Blt_ListItem *)NULL) {
+	panic("can't allocate list item structure");
     }
-    return (entryPtr);
+    itemPtr->keyPtr = key;
+    itemPtr->clientData = (ClientData)NULL;
+    itemPtr->nextPtr = itemPtr->prevPtr = (Blt_ListItem *)NULL;
+    itemPtr->listPtr = (Blt_List *)NULL;
+    return (itemPtr);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_DestroyListEntry --
+ * Blt_FreeItem --
  *
  *	Free the memory allocated for the entry.
  *
@@ -120,16 +122,16 @@ Blt_CreateListEntry(key)
  *----------------------------------------------------------------------
  */
 void
-Blt_DestroyListEntry(entryPtr)
-    Blt_ListEntry *entryPtr;
+Blt_FreeItem(itemPtr)
+    Blt_ListItem *itemPtr;
 {
-    free((char *)entryPtr);
+    free((char *)itemPtr);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_ClearList --
+ * Blt_ResetList --
  *
  *	Removes all the entries from a list, removing pointers to the
  *	objects and keys (not the objects or keys themselves).
@@ -141,24 +143,24 @@ Blt_DestroyListEntry(entryPtr)
  *----------------------------------------------------------------------
  */
 void
-Blt_ClearList(listPtr)
-    Blt_LinkedList *listPtr;	/* List to clear */
+Blt_ResetList(listPtr)
+    Blt_List *listPtr;		/* List to clear */
 {
-    register Blt_ListEntry *oldPtr;
-    register Blt_ListEntry *entryPtr = listPtr->headPtr;
+    register Blt_ListItem *oldPtr;
+    register Blt_ListItem *itemPtr = listPtr->headPtr;
 
-    while (entryPtr != (Blt_ListEntry *)NULL) {
-	oldPtr = entryPtr;
-	entryPtr = entryPtr->nextPtr;
-	Blt_DestroyListEntry(oldPtr);
+    while (itemPtr != (Blt_ListItem *)NULL) {
+	oldPtr = itemPtr;
+	itemPtr = itemPtr->nextPtr;
+	Blt_FreeItem(oldPtr);
     }
-    Blt_InitLinkedList(listPtr, listPtr->type);
+    Blt_InitList(listPtr, listPtr->type);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_DeleteLinkedList
+ * Blt_DeleteList
  *
  *     Frees all list structures
  *
@@ -168,17 +170,17 @@ Blt_ClearList(listPtr)
  *----------------------------------------------------------------------
  */
 void
-Blt_DeleteLinkedList(listPtr)
-    Blt_LinkedList *listPtr;
+Blt_DeleteList(listPtr)
+    Blt_List *listPtr;
 {
-    Blt_ClearList(listPtr);
+    Blt_ResetList(listPtr);
     free((char *)listPtr);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_LinkListAfter --
+ * Blt_LinkAfter --
  *
  *	Inserts an entry following a given entry.
  *
@@ -188,39 +190,40 @@ Blt_DeleteLinkedList(listPtr)
  *----------------------------------------------------------------------
  */
 void
-Blt_LinkListAfter(listPtr, entryPtr, afterPtr)
-    Blt_LinkedList *listPtr;
-    Blt_ListEntry *entryPtr;
-    Blt_ListEntry *afterPtr;
+Blt_LinkAfter(listPtr, itemPtr, afterPtr)
+    Blt_List *listPtr;
+    Blt_ListItem *itemPtr;
+    Blt_ListItem *afterPtr;
 {
     /*
      * If the list keys are strings, change the key to a Tk_Uid
      */
     if (listPtr->type == TCL_STRING_KEYS) {
-	entryPtr->keyPtr = Tk_GetUid(entryPtr->keyPtr);
+	itemPtr->keyPtr = Tk_GetUid(itemPtr->keyPtr);
     }
-    if (listPtr->headPtr == (Blt_ListEntry *)NULL) {
-	listPtr->tailPtr = listPtr->headPtr = entryPtr;
+    if (listPtr->headPtr == (Blt_ListItem *)NULL) {
+	listPtr->tailPtr = listPtr->headPtr = itemPtr;
     } else {
-	if (afterPtr == (Blt_ListEntry *)NULL) {
+	if (afterPtr == (Blt_ListItem *)NULL) {
 	    afterPtr = listPtr->tailPtr;
 	}
-	entryPtr->nextPtr = afterPtr->nextPtr;
-	entryPtr->prevPtr = afterPtr;
+	itemPtr->nextPtr = afterPtr->nextPtr;
+	itemPtr->prevPtr = afterPtr;
 	if (afterPtr == listPtr->tailPtr) {
-	    listPtr->tailPtr = entryPtr;
+	    listPtr->tailPtr = itemPtr;
 	} else {
-	    afterPtr->nextPtr->prevPtr = entryPtr;
+	    afterPtr->nextPtr->prevPtr = itemPtr;
 	}
-	afterPtr->nextPtr = entryPtr;
+	afterPtr->nextPtr = itemPtr;
     }
+    itemPtr->listPtr = listPtr;
     listPtr->numEntries++;
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_LinkListBefore --
+ * Blt_LinkBefore --
  *
  *	Inserts an entry preceding a given entry.
  *
@@ -230,39 +233,40 @@ Blt_LinkListAfter(listPtr, entryPtr, afterPtr)
  *----------------------------------------------------------------------
  */
 void
-Blt_LinkListBefore(listPtr, entryPtr, beforePtr)
-    Blt_LinkedList *listPtr;	/* List to contain new entry */
-    Blt_ListEntry *entryPtr;	/* New entry to be inserted */
-    Blt_ListEntry *beforePtr;	/* Entry to link before */
+Blt_LinkBefore(listPtr, itemPtr, beforePtr)
+    Blt_List *listPtr;		/* List to contain new entry */
+    Blt_ListItem *itemPtr;	/* New entry to be inserted */
+    Blt_ListItem *beforePtr;	/* Entry to link before */
 {
     /*
      * If the list keys are strings, change the key to a Tk_Uid
      */
     if (listPtr->type == TCL_STRING_KEYS) {
-	entryPtr->keyPtr = Tk_GetUid(entryPtr->keyPtr);
+	itemPtr->keyPtr = Tk_GetUid(itemPtr->keyPtr);
     }
-    if (listPtr->headPtr == (Blt_ListEntry *)NULL) {
-	listPtr->tailPtr = listPtr->headPtr = entryPtr;
+    if (listPtr->headPtr == (Blt_ListItem *)NULL) {
+	listPtr->tailPtr = listPtr->headPtr = itemPtr;
     } else {
-	if (beforePtr == (Blt_ListEntry *)NULL) {
+	if (beforePtr == (Blt_ListItem *)NULL) {
 	    beforePtr = listPtr->headPtr;
 	}
-	entryPtr->prevPtr = beforePtr->prevPtr;
-	entryPtr->nextPtr = beforePtr;
+	itemPtr->prevPtr = beforePtr->prevPtr;
+	itemPtr->nextPtr = beforePtr;
 	if (beforePtr == listPtr->headPtr) {
-	    listPtr->headPtr = entryPtr;
+	    listPtr->headPtr = itemPtr;
 	} else {
-	    beforePtr->prevPtr->nextPtr = entryPtr;
+	    beforePtr->prevPtr->nextPtr = itemPtr;
 	}
-	beforePtr->prevPtr = entryPtr;
+	beforePtr->prevPtr = itemPtr;
     }
+    itemPtr->listPtr = listPtr;
     listPtr->numEntries++;
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_UnlinkListEntry --
+ * Blt_UnlinkItem --
  *
  *	Unlinks an entry from the given list. The entry itself is
  *	not deallocated, but only removed from the list.
@@ -273,29 +277,35 @@ Blt_LinkListBefore(listPtr, entryPtr, beforePtr)
  *----------------------------------------------------------------------
  */
 void
-Blt_UnlinkListEntry(listPtr, entryPtr)
-    Blt_LinkedList *listPtr;
-    Blt_ListEntry *entryPtr;
+Blt_UnlinkItem(itemPtr)
+    Blt_ListItem *itemPtr;
 {
-    if (listPtr->headPtr == entryPtr) {
-	listPtr->headPtr = entryPtr->nextPtr;
+    Blt_List *listPtr;
+
+    listPtr = itemPtr->listPtr;
+    if (listPtr != NULL) {
+	if (listPtr->headPtr == itemPtr) {
+	    listPtr->headPtr = itemPtr->nextPtr;
+	}
+	if (listPtr->tailPtr == itemPtr) {
+	    listPtr->tailPtr = itemPtr->prevPtr;
+	}
+	if (itemPtr->nextPtr != NULL) {
+	    itemPtr->nextPtr->prevPtr = itemPtr->prevPtr;
+	}
+	if (itemPtr->prevPtr != NULL) {
+	    itemPtr->prevPtr->nextPtr = itemPtr->nextPtr;
+	}
+	itemPtr->listPtr = NULL;
+	listPtr->numEntries--;
     }
-    if (listPtr->tailPtr == entryPtr) {
-	listPtr->tailPtr = entryPtr->prevPtr;
-    }
-    if (entryPtr->nextPtr != NULL) {
-	entryPtr->nextPtr->prevPtr = entryPtr->prevPtr;
-    }
-    if (entryPtr->prevPtr != NULL) {
-	entryPtr->prevPtr->nextPtr = entryPtr->nextPtr;
-    }
-    listPtr->numEntries--;
 }
 
+#ifdef notdef
 /*
  *----------------------------------------------------------------------
  *
- * Blt_SetListValue --
+ * Blt_SetItemValue --
  *
  *	Sets the entry data pointer to the given value.
  *
@@ -309,17 +319,21 @@ Blt_UnlinkListEntry(listPtr, entryPtr)
  *----------------------------------------------------------------------
  */
 void
-Blt_SetListValue(entryPtr, clientData)
-    Blt_ListEntry *entryPtr;	/* Pointer to the entry */
+Blt_SetItemValue(itemPtr, clientData)
+    Blt_ListItem *itemPtr;	/* Pointer to the entry */
     ClientData clientData;	/* Data attached to key */
 {
-    entryPtr->clientData = clientData;
+    itemPtr->clientData = clientData;
 }
 
+#endif
+
+
+#ifdef notdef
 /*
  *----------------------------------------------------------------------
  *
- * Blt_ListIndex --
+ * Blt_List_Index --
  *
  *	Find the position of an entry in the list.
  *
@@ -330,26 +344,28 @@ Blt_SetListValue(entryPtr, clientData)
  *----------------------------------------------------------------------
  */
 int
-Blt_ListIndex(listPtr, searchPtr)
-    Blt_LinkedList *listPtr;	/* List to search */
-    Blt_ListEntry *searchPtr;	/* Entry to match */
+Blt_ItemIndex(listPtr, searchPtr)
+    Blt_List *listPtr;		/* List to search */
+    Blt_ListItem *searchPtr;	/* Entry to match */
 {
     register int count = 0;
-    register Blt_ListEntry *entryPtr;	/* Entry to match */
+    register Blt_ListItem *itemPtr;	/* Entry to match */
 
-    for (entryPtr = listPtr->headPtr; entryPtr != NULL;
-	entryPtr = entryPtr->nextPtr) {
-	if (searchPtr == entryPtr)
+    for (itemPtr = listPtr->headPtr; itemPtr != NULL;
+	itemPtr = itemPtr->nextPtr) {
+	if (searchPtr == itemPtr)
 	    return (count);
 	count++;
     }
     return (-1);
 }
 
+#endif
+
 /*
  *----------------------------------------------------------------------
  *
- * Blt_FindListEntry --
+ * Blt_FindItem --
  *
  *	Find the first entry matching the key given.
  *
@@ -359,31 +375,31 @@ Blt_ListIndex(listPtr, searchPtr)
  *
  *----------------------------------------------------------------------
  */
-Blt_ListEntry *
-Blt_FindListEntry(listPtr, searchKey)
-    Blt_LinkedList *listPtr;	/* List to search */
+Blt_ListItem *
+Blt_FindItem(listPtr, searchKey)
+    Blt_List *listPtr;		/* List to search */
     char *searchKey;		/* Key to match */
 {
-    register Blt_ListEntry *entryPtr;
+    register Blt_ListItem *itemPtr;
     Tk_Uid newPtr;
 
     newPtr = searchKey;
     if (listPtr->type == TCL_STRING_KEYS) {
 	newPtr = Tk_GetUid(searchKey);
     }
-    for (entryPtr = listPtr->headPtr; entryPtr != NULL;
-	entryPtr = entryPtr->nextPtr) {
-	if (newPtr == entryPtr->keyPtr)
-	    return (entryPtr);
+    for (itemPtr = listPtr->headPtr; itemPtr != NULL;
+	itemPtr = itemPtr->nextPtr) {
+	if (newPtr == itemPtr->keyPtr)
+	    return (itemPtr);
     }
-    return (Blt_ListEntry *) NULL;
+    return (Blt_ListItem *) NULL;
 }
 
 #ifdef notdef
 /*
  *----------------------------------------------------------------------
  *
- * Blt_FirstListEntry --
+ * Blt_FirstListItem --
  *
  *	Find the first entry in the list and return its pointer.
  *	In addition, update the given search pointer.
@@ -395,19 +411,19 @@ Blt_FindListEntry(listPtr, searchKey)
  *
  *----------------------------------------------------------------------
  */
-Blt_ListEntry *
-Blt_FirstListEntry(listPtr, entryPtrPtr)
-    Blt_LinkedList *listPtr;	/* The list we are searching */
-    Blt_ListEntry **entryPtrPtr;/* Search pointer to set */
+Blt_ListItem *
+Blt_FirstListItem(listPtr, itemPtrPtr)
+    Blt_List *listPtr;		/* The list we are searching */
+    Blt_ListItem **itemPtrPtr;	/* Search pointer to set */
 {
-    if (listPtr == (Blt_LinkedList *)NULL)
-	return (Blt_ListEntry *) NULL;
+    if (listPtr == (Blt_List *)NULL)
+	return (Blt_ListItem *) NULL;
 
-    if (listPtr->headPtr == (Blt_ListEntry *)NULL)
-	return (Blt_ListEntry *) NULL;
+    if (listPtr->headPtr == (Blt_ListItem *)NULL)
+	return (Blt_ListItem *) NULL;
 
-    if (entryPtrPtr != (Blt_ListEntry **)NULL) {
-	*entryPtrPtr = listPtr->headPtr;
+    if (itemPtrPtr != (Blt_ListItem **)NULL) {
+	*itemPtrPtr = listPtr->headPtr;
     }
     return (listPtr->headPtr);
 }
@@ -415,7 +431,7 @@ Blt_FirstListEntry(listPtr, entryPtrPtr)
 /*
  *----------------------------------------------------------------------
  *
- * Blt_ListLastEntry --
+ * Blt_LastListItem --
  *
  *	Find the last entry in the list using the given entry as
  *	a cursor to the last entry and return its pointer.
@@ -428,22 +444,22 @@ Blt_FirstListEntry(listPtr, entryPtrPtr)
  *
  *----------------------------------------------------------------------
  */
-Blt_ListEntry *
-Blt_ListLastEntry(entryPtrPtr)
-    Blt_ListEntry **entryPtrPtr;/* Search pointer of current position */
+Blt_ListItem *
+Blt_LastListItem(itemPtrPtr)
+    Blt_ListItem **itemPtrPtr;	/* Search pointer of current position */
 {
-    if ((entryPtrPtr == (Blt_ListEntry **)NULL) ||
-	(*entryPtrPtr == (Blt_ListEntry *)NULL)) {
-	return (Blt_ListEntry *) NULL;
+    if ((itemPtrPtr == (Blt_ListItem **)NULL) ||
+	(*itemPtrPtr == (Blt_ListItem *)NULL)) {
+	return (Blt_ListItem *) NULL;
     }
-    *entryPtrPtr = (*entryPtrPtr)->prevPtr;
-    return (*entryPtrPtr);
+    *itemPtrPtr = (*itemPtrPtr)->prevPtr;
+    return (*itemPtrPtr);
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * Blt_NextListEntry --
+ * Blt_NextItem --
  *
  *	Find the next entry in the list using the given search pointer
  *	as the current location and return its pointer.
@@ -456,22 +472,22 @@ Blt_ListLastEntry(entryPtrPtr)
  *
  *----------------------------------------------------------------------
  */
-Blt_ListEntry *
-Blt_NextListEntry(entryPtrPtr)
-    Blt_ListEntry **entryPtrPtr;/* Search pointer indicates current position */
+Blt_ListItem *
+Blt_NextItem(itemPtrPtr)
+    Blt_ListItem **itemPtrPtr;	/* Search pointer indicates current position */
 {
-    if ((entryPtrPtr == NULL) || (*entryPtrPtr == NULL)) {
-	return (Blt_ListEntry *) NULL;
+    if ((itemPtrPtr == NULL) || (*itemPtrPtr == NULL)) {
+	return (Blt_ListItem *) NULL;
     }
-    *entryPtrPtr = (*entryPtrPtr)->nextPtr;
-    return (*entryPtrPtr);
+    *itemPtrPtr = (*itemPtrPtr)->nextPtr;
+    return (*itemPtrPtr);
 }
 
 #endif
 /*
  *----------------------------------------------------------------------
  *
- * Blt_DeleteListEntry --
+ * Blt_DeleteItem --
  *
  *	Find the entry and free the memory allocated for the entry.
  *
@@ -481,10 +497,9 @@ Blt_NextListEntry(entryPtrPtr)
  *----------------------------------------------------------------------
  */
 void
-Blt_DeleteListEntry(listPtr, entryPtr)
-    Blt_LinkedList *listPtr;
-    Blt_ListEntry *entryPtr;
+Blt_DeleteItem(itemPtr)
+    Blt_ListItem *itemPtr;
 {
-    Blt_UnlinkListEntry(listPtr, entryPtr);
-    Blt_DestroyListEntry(entryPtr);
+    Blt_UnlinkItem(itemPtr);
+    Blt_FreeItem(itemPtr);
 }
