@@ -26,10 +26,53 @@
  */
 
 #ifndef lint
-static char rcsid[] = "/usr/home/gah/repository/blt/tkAppInit.c,v 1.2 1994/01/18 21:21:53 gah Exp SPRITE (Berkeley) $Revision";
+static char rcsid[] = "$Header: /usr/home/gah/repository/blt/tkAppInit.c,v 1.3 1994/04/02 04:37:26 gah Exp $ SPRITE (Berkeley) $Revision";
 #endif /* not lint */
 
 #include "tk.h"
+#include "blt.h"
+
+#if (TK_MAJOR_VERSION > 3)
+/*
+ * The following variable is a special hack that is needed in order for
+ * Sun shared libraries to be used for Tcl.
+ */
+
+#ifdef NEED_MATHERR
+extern int matherr();
+int *tclDummyMathPtr = (int *) matherr;
+#endif
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * main --
+ *
+ *	This is the main program for the application.
+ *
+ * Results:
+ *	None: Tk_Main never returns here, so this procedure never
+ *	returns either.
+ *
+ * Side effects:
+ *	Whatever the application does.
+ *
+ *----------------------------------------------------------------------
+ */
+extern int Tcl_AppInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+int
+main(argc, argv)
+    int argc;			/* Number of command-line arguments. */
+    char **argv;		/* Vector of command-line arguments. */
+{
+
+    Tk_Main(argc, argv, Tcl_AppInit);
+    return 0;			/* Needed only to prevent compiler warning. */
+}
+
+#else
 
 /*
  * The following variable is a special hack that allows applications
@@ -40,6 +83,8 @@ static char rcsid[] = "/usr/home/gah/repository/blt/tkAppInit.c,v 1.2 1994/01/18
 
 extern int main();
 int *tclDummyMainPtr = (int *) main;
+
+#endif /* TK_MAJOR_VERSION >= 4 */
 
 /*
  *----------------------------------------------------------------------
@@ -64,10 +109,12 @@ int
 Tcl_AppInit(interp)
     Tcl_Interp *interp;		/* Interpreter for application. */
 {
-    Tk_Window main;
-
-    main = Tk_MainWindow(interp);
-
+    if (Tcl_Init(interp) == TCL_ERROR) {
+	return TCL_ERROR;
+    }
+    if (Tk_Init(interp) == TCL_ERROR) {
+	return TCL_ERROR;
+    }
     /*
      * Call the init procedures for included packages.  Each call should
      * look like this:
@@ -81,19 +128,8 @@ Tcl_AppInit(interp)
     if (Blt_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
-#ifdef ITCL
-    if (Itcl_Init(interp) == TCL_ERROR) {
-	return TCL_ERROR;
-    }
-#endif
-    if (Tcl_Init(interp) == TCL_ERROR) {
-	return TCL_ERROR;
-    }
-    if (Tk_Init(interp) == TCL_ERROR) {
-	return TCL_ERROR;
-    }
-
     /*
+
      * Call Tcl_CreateCommand for application-specific commands, if
      * they weren't already created by the init procedures called above.
      */
@@ -105,6 +141,10 @@ Tcl_AppInit(interp)
      * then no user-specific startup file will be run under any conditions.
      */
 
+#if (TCL_MAJOR_VERSION >= 7) && (TCL_MINOR_VERSION >= 5)
+    Tcl_SetVar(interp, "tcl_rcFileName", "~/.wishrc", TCL_GLOBAL_ONLY);
+#else
     tcl_RcFileName = "~/.wishrc";
+#endif
     return TCL_OK;
 }
